#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#4.0.264 - version update

import sys
import os
import codecs
from fnmatch import fnmatch
import common as cf
from copy import deepcopy

product = os.path.splitext(os.path.basename(sys.argv[0]))[0]
realcmd = os.path.splitext(os.path.basename(os.path.realpath(sys.argv[0])))[0]
if os.name == 'posix':
	import pwd
	runas = os.environ['SUDO_USER'] if os.geteuid() == 0 else pwd.getpwuid(os.geteuid()).pw_name
else:
	runas = os.environ['USERNAME']

defaults = {
	'setup': {
		'version': u'4.0.264',
		'product': product,
		'inifile': os.path.abspath(os.sep.join(['.','etc','config.ini'])),
		'graphs': os.path.abspath(os.sep.join(['.','graphs'])),
		'plugins': os.path.abspath(os.sep.join(['.','plugins'])),
		'active': '',
		},
	'filtergraph': {
		'rootfolder': os.path.abspath('..'),
		},
	}

def getproductdefaults():
	global defaults
	# pointer to custom default folder in posix (linux)
	if os.name == 'posix' and os.path.exists(os.sep.join(['','usr','local','etc',realcmd,''.join([product,'.ini'])])):
		defaults['setup']['inifile'] = os.sep.join(['','usr','local','etc',realcmd,''.join([product,'.ini'])])
	# pointer to custom default folder in windows
	elif os.name == 'nt' and os.path.exists(os.sep.join([os.environ['ALLUSERSPROFILE'],'Application Data',realcmd,''.join([product,'.ini'])])):
		defaults['setup']['inifile'] = os.sep.join([os.environ['ALLUSERSPROFILE'],'Application Data',realcmd,''.join([product,'.ini'])])
	else:
		defaults['setup']['inifile'] = expanduserdir(defaults['setup']['inifile'])
	if not os.path.exists(defaults['setup']['inifile']):
		raise RuntimeError("Error: corrupted installation - missing master config.ini file")
	if not os.path.exists(defaults['setup']['graphs']):
		raise RuntimeError("Error: corrupted installation - missing master graphs folder")
	if not os.path.exists(defaults['setup']['plugins']):
		raise RuntimeError("Error: corrupted installation - missing master plugins folder")
	# return copy of defaults
	d = {}
	for k in defaults:
		d[k] = {}
		d[k].update(defaults[k])
	return d

def updateconfig(d,mastergraphs='',masterplugins=''):
	'''update config values from inifile'''

	# create default .ini file if missing
	_makedefaultini(defaults['setup']['inifile'],d)

	# set runas user
	lrunas = d['server']['runas'] if 'server' in d and 'runas' in d['server'] and d['server']['runas'] else runas

	# retrieve .ini file using runas value
	inistring = readrawinistring(expanduserdir(d['setup']['inifile'],lrunas))

	# update user options
	useropts = getuseroptions(inistring)
	inistring = setuseroptions(inistring,useropts)

	# convert to dictionary
	e = iniString2Dict(inistring)

	# update main config
	for k in e:
		if not k in d: d[k] = {}
		d[k].update(e[k])

	if masterplugins and mastergraphs:

		workinggraphs = expanduserdir(d['setup']['graphs'],lrunas)
		workingplugins = expanduserdir(d['setup']['plugins'],lrunas)

		# copy master plugins plugins config folder
		_updatedir([masterplugins],workingplugins,lrunas)

		# test for "graphs-ce" and "graphs-pro"
		suffix = '-%s'%(d['setup']['graphs'].split('-')[-1].lower()) if d['setup']['graphs'].split('-')[-1].lower() in ['ce','pro'] else ''
		if suffix:
			activegraphs = ''.join([mastergraphs,suffix])
			if not os.path.exists(activegraphs):
				raise RuntimeError("Error: corrupted installation - missing master graphs folder"%())
			_updatedir([mastergraphs,activegraphs],workinggraphs,lrunas)
		else:
			_updatedir([mastergraphs],workinggraphs,lrunas)

def getinstalledgraphs(graphs,graphpatterns=['*']):
	graphnames = []
	for graphpattern in graphpatterns:
		graphnames.extend([graphname for graphname in os.listdir(graphs) if fnmatch(graphname,graphpattern) and os.path.exists(os.sep.join([graphs,graphname,'config.ini']))])
	return sorted(cf.dedupelist(graphnames))

def readrawinistring(inifile):
	return codecs.open(inifile,'r','utf8').read()

def writerawinistring(inifile,inistring):
	codecs.open(inifile,'w','utf-8').write(inistring)

def expanduserdir(path,lrunas=runas):
	# set "path" to proper user's $HOME folder when invoked as normal or super user
	if path.startswith('~'+os.sep) and os.geteuid() == 0 and os.name == 'posix':
		return path.replace('/',os.sep).replace('\\',os.sep).replace('~'+os.sep,pwd.getpwnam(lrunas).pw_dir+os.sep)
	else:
		return os.path.abspath(os.path.expanduser(path.replace('/',os.sep).replace('\\',os.sep)))

def getuseroptions(inistring):
	# strip ['USER'] section from inistring and convert to a list for search/replace
	isuser = False
	useropts = []
	for line in [[line[0].strip(),line[1].strip()] for line in [[line]+[''] if len(line.split('=',1)) < 2 else line.split('=',1) for line in inistring.splitlines()]]:
		if line[0] == '[USER]':
			isuser = True
		elif line[0].startswith('[') and line[0].endswith(']') and isuser:
			break
		elif isuser and line[0] and not line[0].startswith(';') and not line[0].startswith('#'):
			useropts.append([line[0].lower(),line[1]])
	return useropts

def setuseroptions(inistring='',useropts=[]):
	config = []
	for line in [[line[0].strip(),line[1].strip()] for line in [[line]+[''] if len(line.split('=',1)) < 2 else line.split('=',1) for line in inistring.splitlines()]]:
		if not line[0].strip().startswith(';') and not line[0].strip().startswith('#'):
			for i,j in useropts:
				line[0] = line[0].replace('%%(%s)s'%(str(i)),str(j))
				line[1] = line[1].replace('%%(%s)s'%(str(i)),str(j))
		config.append('='.join(line).rstrip('='))
	return '\x0d\x0a'.join(config)+'\x0d\x0a'

def Dict2iniString(config):
	inistring = []
	for k in sorted(config.keys()):
		inistring.append(k.join(['[',']']))
		for kk in sorted(config[k].keys()):
			inistring.append('='.join([str(kk),str(config[k][kk])]))
		inistring.append('')
	return '\x0d\x0a'.join(inistring)+'\x0d\x0a'

def iniString2Dict(inistring):
	config = {}
	section = ''
	for line in [[line[0].strip(),line[1].strip()] for line in [[line]+[''] if len(line.split('=',1)) < 2 else line.split('=',1) for line in inistring.splitlines() if ('=' in line or (line.strip().startswith('[') and line.strip().endswith(']'))) and not line.strip().startswith(';') and not line.strip().startswith('#')]]:
		if line[0].startswith('[') and line[0].endswith(']'):
			section = line[0].strip('][')
			config[section] = {}
			continue
		if not section in config: continue
		config[section][line[0].lower()] = line[1]
	return config

def parseconfig(conf):
	errors = []
	cfg = {}
	config = deepcopy(conf)
	roottypes = ['BUILDS','CORPORA','ENGINES','RAW','TRAININGS','TRANSLATIONS','CORPORA.demo','RAW.demo',]

	# *** test for required sections ***
	if 'setup' in config:
		if not 'graphs' in config['setup']:
			errors.append([__name__,'missing','\"[setup] graphs=\" value not found'])
		if not 'plugins' in config['setup']:
			errors.append([__name__,'missing','\"[setup] plugins=\" value not found'])
	else:
		errors.append([__name__,'missing','\"[setup]\" section not found'])

	if not 'manager' in config:
		errors.append([__name__,'missing','\"[setup] graphs=\" value not found'])

	if 'filtergraph' in config:
		if not 'graphname' in config['filtergraph']:
			errors.append([__name__,'missing','\"[filtergraph] graphname=\" value not found'])
	else:
		errors.append([__name__,'missing','\"[filtergraph] section not found'])

	try:
		reader = ','.join([v.strip() for v in [k for k in config.keys() if ',reader-' in k.replace(' ','')][0].split(',')])
	except IndexError,e:
		errors.append([__name__,'missing','\"[reader]\" section not found'])
	else:
		if not 'stage' in config[reader]:
			errors.append([__name__,'missing','\"[%s] stage=\" value not found'%(reader)])
		if not 'roottype' in config[reader]:
			errors.append([__name__,'missing','\"[%s] roottype=\" value not found'%(reader)])
		if 'roottype' in config[reader] and not config[reader]['roottype'] in roottypes:
			errors.append([__name__,'invalid','\"[%s] roottype=\" value must be %s'%(reader,' or '.join(roottypes))])

	try:
		writer = ','.join([v.strip() for v in [k for k in config.keys() if ',writer-' in k.replace(' ','')][0].split(',')])
		if not 'stage' in config[writer]:
			errors.append([__name__,'missing','\"[%s] stage=\" value not found'%(writer)])
	except IndexError,e:
		errors.append([__name__,'missing','\"[writer]\" section not found'])
	else:
		if not 'roottype' in config[writer]:
			errors.append([__name__,'missing','\"[%s] roottype=\" value not found'%(writer)])
		if 'roottype' in config[reader] and not config[writer]['roottype'] in roottypes:
			errors.append([__name__,'invalid','\"[%s] roottype=\" value must be %s'%(writer,' or '.join(roottypes))])

	if errors: return errors

	# 0) get root [USER] 
	for k in [k for k in config.keys() if k == 'USER']:
		cfg[k] = dict(config[k])
		del(config[k])
		break

	# 1) get root [setup]
	for k in [k for k in config.keys() if k == 'setup']:
		cfg[k] = dict(config[k])
		del(config[k])
		break

	# 2) get root [manager] 
	for k in [k for k in config.keys() if k == 'manager']:
		cfg[k] = dict(config[k])
		del(config[k])
		break

	# 3) get [filtergraph]
	for k in [k for k in config.keys() if k == 'filtergraph']:
		cfg['manager'][k] = dict(config[k])
		del(config[k])
		cfg['manager'][k]['aligner'] = {}
		cfg['manager'][k]['filter'] = {}
		cfg['manager'][k]['postfilter'] = {}
		cfg['manager'][k]['stage'] = cfg['manager'][k]['stage'].strip(',').split(',') if 'stage' in cfg['manager'][k] else []
		cfg['manager'][k]['superdomains'] = cfg['manager'][k]['superdomains'].strip(',').split(',') if 'superdomains' in cfg['manager'][k] else []
		cfg['manager'][k]['domains'] = cfg['manager'][k]['domains'].strip(',').split(',') if 'domains' in cfg['manager'][k] else []
		cfg['manager'][k]['subdomains'] = cfg['manager'][k]['subdomains'].strip(',').split(',') if 'subdomains' in cfg['manager'][k] else []
		cfg['manager'][k]['corpustypes'] = [l.strip() for l in cfg['manager'][k]['corpustypes'].strip(',').split(',') if l.strip().startswith('lm') or l.strip().startswith('tm') or l.strip() == '*'] if 'corpustypes' in cfg['manager'][k] else []
		cfg['manager'][k]['sourcelanguages'] = cfg['manager'][k]['sourcelanguages'].strip(',').split(',') if 'sourcelanguages' in cfg['manager'][k] else []
		cfg['manager'][k]['languages'] = cfg['manager'][k]['languages'].strip(',').split(',') if 'languages' in cfg['manager'][k] else []
		cfg['manager'][k]['languages'] = cf.dedupelist(cfg['manager'][k]['sourcelanguages']+cfg['manager'][k]['languages'])
		cfg['manager'][k]['languagepairs'] = [pair.strip(' -').strip().split('-') for pair in cfg['manager'][k]['languagepairs'].strip(',').split(',')] if 'languagepairs' in cfg['manager'][k] else []
		cfg['manager'][k]['languagepairs'] = [tuple([src.lower(),tgt.lower()]) for src,tgt in [pair if len(pair) == 2 else pair+pair for pair in cfg['manager'][k]['languagepairs']]]
		cfg['manager'][k]['filespec'] = cfg['manager'][k]['filespec'].strip(',').split(',') if 'filespec' in cfg['manager'][k] else []
		cfg['manager'][k]['graphs'] = cfg['manager'][k]['graphs'] if 'graphs' in cfg['manager'][k] and cfg['manager'][k]['graphs'] else cfg['setup']['graphs']
		cfg['manager'][k]['plugins'] = cfg['manager'][k]['plugins'] if 'plugins' in cfg['manager'][k] and cfg['manager'][k]['plugins'] else cfg['setup']['plugins']
		break

	# 4) get [reader]
	cfg['manager']['filtergraph']['reader'] = {reader: dict(config[reader]),}
	# parse known options to lists
	for k in [k for k in cfg['manager']['filtergraph']['reader'][reader].keys() if cf.isinteger(k[-1])]:
		if k[:4] == 'args' and cf.isinteger(k[4:]):
			cfg['manager']['filtergraph']['reader'][reader]['args'][int(k[4:])] = cfg['manager']['filtergraph']['reader'][reader][k]
			del(cfg['manager']['filtergraph']['reader'][reader][k])
		elif k[:7] == 'engines' and cf.isinteger(k[7:]):
			if not ',' in cfg['manager']['filtergraph']['reader'][reader][k]: continue
			cfg['manager']['filtergraph']['reader'][reader]['engines'][int(k[7:])] = cfg['manager']['filtergraph']['reader'][reader][k]
			del(cfg['manager']['filtergraph']['reader'][reader][k])
	del(config[reader])

	# 5) get [writer]
	cfg['manager']['filtergraph']['writer'] = {writer: dict(config[writer]),}
	# parse known options to lists
	for k in [k for k in cfg['manager']['filtergraph']['writer'][writer].keys() if cf.isinteger(k[-1])]:
		if k[:4] == 'args' and cf.isinteger(k[4:]):
			cfg['manager']['filtergraph']['writer'][writer]['args'][int(k[4:])] = cfg['manager']['filtergraph']['writer'][writer][k]
			del(cfg['manager']['filtergraph']['writer'][writer][k])
		elif k[:7] == 'engines' and cf.isinteger(k[7:]):
			if not ',' in cfg['manager']['filtergraph']['writer'][writer][k]: continue
			cfg['manager']['filtergraph']['writer'][writer]['engines'][int(k[7:])] = cfg['manager']['filtergraph']['writer'][writer][k]
			del(cfg['manager']['filtergraph']['writer'][writer][k])
	del(config[writer])

	# 6) fill plugin configurations
	for k in [k for k in config.keys() if len(k.split(',')) == 1 and not 'input' in k]:
		lang = k.replace('-','_')
		for plugintype in ['aligner','filter','postfilter']:
			cfg['manager']['filtergraph'][plugintype][lang] = {}
			for kk in [kk for kk in config[k].keys() if kk.startswith(plugintype) and cf.isinteger(kk[len(plugintype):])]:
				if ','.join([k,kk.replace(plugintype,''),config[k][kk]]) in config.keys():
					cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))] = {config[k][kk]:config[','.join([k,kk.replace(plugintype,''),config[k][kk]])],}
					del(config[','.join([k,kk.replace(plugintype,''),config[k][kk]])])
					# parse known options to lists
					for kkk in cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]].keys():
						if kkk in ['bufsize','close_fds','creationflags','cwd','env','executable','preexec_fn','shell','startupinfo','stderr','stdin','stdout','universal_newlines',]:
							if not 'kw' in cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]: cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]['kw'] = {}
							cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]['kw'][kkk] = cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk]
							del(cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk])
						elif cf.isinteger(kkk[4:]) and kkk[:4] == 'args':
							if not 'args' in cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]: cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]['args'] = {}
							cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]['args'][int(kkk[4:])] = cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk]
							del(cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk])
						elif cf.isinteger(kkk[7:]) and kkk[:7] == 'engines':
							if not ',' in cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk]: continue
							if not 'engines' in cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]: cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]['engines'] = {}
							cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]['engines'][int(kkk[7:])] = cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk]
							del(cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk])
				else:
					cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))] = {config[k][kk]:{},}
		del(config[k])

	# 7) add toolchain for other languages
	if 'languages' in cfg['manager']['filtergraph']:
		for k in cfg['manager']['filtergraph']['languages']:
			lang = '' if '*' in k or '?' in k else k.replace('-','_')
			for plugintype in ['aligner','filter','postfilter']:
				if lang and not lang in cfg['manager']['filtergraph'][plugintype]: cfg['manager']['filtergraph'][plugintype][lang] = {}

#	print _display(cfg)
#	exit()
	return cfg

def _display(cfg):
	for k in sorted(cfg.keys()):
		if type(cfg[k]) is dict:
			print str(k).join(['[',']'])
			for kk in sorted(cfg[k].keys()):
				if type(cfg[k][kk]) is dict:
					print str(kk).join(['     [',']'])
					for kkk in sorted(cfg[k][kk].keys()):
						if type(cfg[k][kk][kkk]) is dict:
							print str(kkk).join(['          [',']'])
							for kkkk in sorted(cfg[k][kk][kkk].keys()):
								if type(cfg[k][kk][kkk][kkkk]) is dict:
									print str(kkkk).join(['               [',']'])
									for kkkkk in sorted(cfg[k][kk][kkk][kkkk].keys()):
										if type(cfg[k][kk][kkk][kkkk][kkkkk]) is dict:
											print str(kkkkk).join(['                    [',']'])
											for kkkkkk in sorted(cfg[k][kk][kkk][kkkk][kkkkk].keys()):
												if type(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk]) is dict:
													print str(kkkkkk).join(['                         [',']'])
													for kkkkkkk in sorted(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk].keys()):
														if type(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk][kkkkkkk]) is dict:
															print str(kkkkkkk).join(['                              [',']'])
															for kkkkkkkk in sorted(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk][kkkkkkk].keys()):
																if type(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk][kkkkkkk][kkkkkkkk]) is dict:
																	print str(kkkkkkkk).join(['                                   [',']'])
																else:
																	print '                                  ','='.join([str(kkkkkkkk),str(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk][kkkkkkk][kkkkkkkk])])
														else:
															print '                             ','='.join([str(kkkkkkk),str(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk][kkkkkkk])])
												else:
													print '                        ','='.join([str(kkkkkk),str(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk])])
										else:
											print '                   ','='.join([str(kkkkk),str(cfg[k][kk][kkk][kkkk][kkkkk])])
								else:
									print '              ','='.join([str(kkkk),str(cfg[k][kk][kkk][kkkk])])
						else:
							print '         ','='.join([str(kkk),str(cfg[k][kk][kkk])])
				else:
					print '    ','='.join([str(kk),str(cfg[k][kk])])
		else:
			print ' ','='.join([str(k),str(cfg[k])])
	print

def _updatedir(masters,working,lrunas):

	if os.path.exists(working):

		workingupdate = '20000101T000000+0000'
		if os.path.exists(os.sep.join([working,'.lastupdate'])):
			workingupdate = open(os.sep.join([working,'.lastupdate']),'r').readline().strip()
		workingdate = cf.str2datetime(workingupdate)

		masterlastupdate = '20000101T000000+0000'
		masterupdates = []
		for master in masters:
			masterupdate = open(os.sep.join([master,'.lastupdate']),'r').readline().strip()
			masterdate = cf.str2datetime(masterupdate)
			if masterdate > workingdate: masterupdates.append(master)
			if masterdate > cf.str2datetime(masterlastupdate): masterlastupdate = masterupdate

		if masterupdates:
			_renameuserdir(working,masterlastupdate)
			_copy(masters,working,lrunas=lrunas)

	else:
		_copy(masters,working,lrunas=lrunas)

def _renameuserdir(working,masterupdate):
	# rename existing folder and copy new one
	content = [	'This folder was saved when CorpusFiltergraph updated',
			'GRAPHS. If you customized this GRAPH, you need to migrate',
			'your changes to the new version. If no changes were made,',
			'you may delete this folder safely.']
	try:
		with open(os.sep.join([working.rstrip(os.sep),'info']),'w') as f:
			f.write('\n'.join(content)+'\n')
	except:
		pass
	while os.path.exists('-'.join([working,masterupdate])):
		masterupdate = masterupdate.split('+')
		masterdatetime = masterupdate[0].split('T')
		masterdatetime[1] = int(masterdatetime[1]) + 1
		masterupdate = '%sT%s+%s'%(masterdatetime[0],masterdatetime[1],masterupdate[1])
	try:
		os.rename(working,'-'.join([working,masterupdate]))
	except:
		pass

def _copy(masters,working,lrunas):
	masterlastupdate = '20000101T000000+0000'
	for master in masters:
		masterupdate = open(os.sep.join([master,'.lastupdate']),'r').readline().strip()
		masterdate = cf.str2datetime(masterupdate)
		if masterdate > cf.str2datetime(masterlastupdate): masterlastupdate = masterupdate
		_copytree(master,working,lrunas)
	with open(os.sep.join([working,'.lastupdate']),'w') as f:
		f.write('%s\n'%(masterlastupdate))

def _makedefaultini(inputini,config):
	if not 'setup' in config.keys() or not 'inifile' in config['setup'].keys():
		config = {}
		for k in defaults:
			config[k] = dict(defaults[k])
	lrunas = config['server']['runas'] if 'server' in config and 'runas' in config['server'] and config['server']['runas'] else runas
	outputini = expanduserdir(config['setup']['inifile'],lrunas)
	if os.path.exists(inputini) and not os.path.exists(outputini):
		content = [line.strip().replace('/',os.sep).replace('\\',os.sep) for line in codecs.open(inputini,'r','utf-8').readlines() if not line.strip().startswith('inifile=')]
		# create output .ini file
		try:
			os.makedirs(os.path.dirname(outputini))
		except OSError,e:
			if not e.errno == 17: raise OSError('Error: failed to create directory %s'%(os.path.dirname(outputini)))
		try:
			codecs.open(outputini,'w','utf-8').write('\n'.join(content)+'\n')
		except:
			raise OSError('Error: failed to open %s'%(outputini))
		if os.name == 'posix' and os.geteuid() == 0:
			cf.chmodrecurse(os.path.dirname(working),pwd.getpwnam(lrunas).pw_uid,pwd.getpwnam(lrunas).pw_gid)
	if os.path.exists(os.sep.join([os.path.dirname(inputini),'environment'])) and not os.path.exists(os.sep.join([os.path.dirname(outputini),'environment'])):
		# create 'environment' file (for linux Bash only, unnecessary after porting all shell scripts to python/domy)
		content = [line.strip().replace('/',os.sep).replace('\\',os.sep) for line in codecs.open(os.sep.join([os.path.dirname(inputini),'environment']),'r','utf-8').readlines() if line.strip() and not line.strip().startswith('export GRAPHS=') and not line.strip().startswith('export PLUGINS=')]
		content.append('='.join(['export GRAPHS',config['setup']['graphs']]))
		content.append('='.join(['export PLUGINS',config['setup']['plugins']]))
		try:
			os.makedirs(os.path.dirname(outputini))
		except OSError,e:
			if not e.errno == 17: raise OSError('Error: failed to create directory %s'%(os.path.dirname(outputini)))
		try:
			codecs.open(os.sep.join([os.path.dirname(outputini),'environment']),'w','utf-8').write('\n'.join(content)+'\n')
		except:
			raise OSError('Error: failed to open %s'%(os.sep.join([os.path.dirname(outputini),'environment'])))
		if os.name == 'posix' and os.geteuid() == 0:
			cf.chmodrecurse(os.path.dirname(working),pwd.getpwnam(lrunas).pw_uid,pwd.getpwnam(lrunas).pw_gid)

def _copytree(src, dst, symlinks=False, ignore=None, lrunas=''):
	"""Recursively copy a directory tree even if directory exists.
	shutil.copytree() modified with os.makedirs()"""
	import shutil
	names = os.listdir(src)
	if ignore is not None:
		ignored_names = ignore(src, names)
	else:
		ignored_names = set()
	try:
		os.makedirs(dst)
	except OSError,e:
		if not e.errno == 17: raise OSError('Error: failed to create directory %s'%(dst))
	errors = []
	for name in names:
		if name in ignored_names:
			continue
		srcname = os.path.join(src, name)
		dstname = os.path.join(dst, name)
		try:
			if symlinks and os.path.islink(srcname):
				linkto = os.readlink(srcname)
				os.symlink(linkto, dstname)
			elif os.path.isdir(srcname):
				_copytree(srcname, dstname, symlinks, ignore)
			else:
				shutil.copy2(srcname, dstname)
			# XXX What about devices, sockets etc.?
		except (IOError, os.error), why:
			errors.append((srcname, dstname, str(why)))
		# catch the Error from the recursive copytree so that we can
		# continue with other files
		except Error, err:
			errors.extend(err.args[0])

	try:
		shutil.copystat(src, dst)
	except OSError, why:
		if WindowsError is not None and isinstance(why, WindowsError):
			# Copying file access times may fail on Windows
			pass
		else:
			errors.extend((src, dst, str(why)))
	if errors:
		raise Error, errors
	if os.name == 'posix' and os.geteuid() == 0 and lrunas:
		cf.chmodrecurse(dst,pwd.getpwnam(lrunas).pw_uid,pwd.getpwnam(lrunas).pw_gid)

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import parser\n\tfrom %s import manager\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0],
	os.path.splitext(os.path.basename(sys.argv[0]))[0],
	)

licensetxt=u'''CorpusFiltergraph™ v4.0
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
