#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#4.0.264 - version update

import sys
import os
import time
import subprocess
from glob import glob
from fnmatch import fnmatch
import tempfile

(fs,root,rttype,stage,supdom,dom,subdom) = range(0,7)
(basename,rdrlang,srclang,kind) = range(-1,-5,-1)

def updatefingerprint(i,o,forceupdate=False,infodata={}):

	def getcontent(fname):
		return open(fname,'r').read().splitlines()

	def setcontent(fname,content):
		f = open(fname,'w')
		f.write('\x0a'.join(content)+'\x0a')
		f.close()

	def getfingerprint(fname):
		tz = timestamp().split('+')[1]
		return {
				'start': '+'.join([time.strftime("%Y-%m-%d %H:%M:%S.999",time.gmtime(os.path.getmtime(fname))),tz]),
				'misc': {},
				'fp': [crc(fname),os.path.getsize(fname),fname],
				'finish': '+'.join([time.strftime("%Y-%m-%d %H:%M:%S.999",time.gmtime(os.path.getmtime(fname))),tz]),
				}

	def parsefingerprint(fname,filefp,infocontent,forceupdate,errmsg='Error: unknown on %s'):

		errors = []
		parsedcontent = []
		test = {
			'start': '',
			'misc': {},
			'fp': [0,0,''],
			'finish': 'unknown time',
			}

		for fileinfo in [fileinfo.split(u'¶') for fileinfo in u'¶'.join(infocontent).replace(u'¶start: ','\nstart: ').splitlines()]:

			# ignore invalid formatted lines
			if not len(fileinfo) > 2:
				parsedcontent.extend(fileinfo)
				continue
			valid = fileinfo[0].startswith('start: ')
			valid = len(fileinfo[-2].split()) == 3 and valid
			valid = fileinfo[-1].startswith('finish: ') and valid
			if not valid:
				parsedcontent.extend(fileinfo)
				continue

			# convert to integers
			try:
				crc,size,filepath = [[int(crc[0]),int(crc[1]),crc[2]] for crc in [fileinfo[-2].split()]][0]
			except:
				parsedcontent.extend(fileinfo)
				continue

			# continue with the next fp in info file
			if not fname == filepath:
				parsedcontent.extend(fileinfo)
				continue

			# fp found in infocontent, check file size because it's fast
			if not size == filefp['fp'][1] and not forceupdate:
				# error, file fingerprint changed after first time creating info file
				errors.append(errmsg%(test['finish']))
				continue

			# check file crc, slower
			if not crc == filefp['fp'][0] and not forceupdate:
				# error, file fingerprint changed after first time creating info file
				errors.append(errmsg%(test['finish']))
				continue

			test['start'] = fileinfo[0].split(':',1)[1].strip()
			for line in fileinfo[1:-2]:
				if '=' in line:
					k,v = line.split('=',1)
				else:
					k,v = 'no label',line
				test['misc'][k.strip()] = v.strip()
			test['fp'] = [filefp['fp'][0],filefp['fp'][1],filepath]
			test['finish'] = fileinfo[-1].split(':',1)[1].strip()

			# add new fingerprint information
			for k in [k for k in filefp['misc'] if 'misc' in filefp]:
				if not k in test['misc']: test['misc'][k] = filefp['misc'][k]

			parsedcontent.extend(setfingerprint(test))

		# fingerprint not in info file
		if test['fp'] == [0,0,'']:
			# add new fingerprint
			parsedcontent.extend(setfingerprint(filefp))

		return parsedcontent,errors

	def setfingerprint(ioio):
		fileinfo = []
		fileinfo.append('%s: %s'%('start',ioio['start']))
		for k in [k for k in sorted(ioio['misc'].keys()) if 'misc' in ioio]:
			fileinfo.append('%s=%s'%(k,ioio['misc'][k]))
		fileinfo.append('%d %d %s'%tuple(ioio['fp']))
		fileinfo.append('%s: %s'%('finish',ioio['finish']))
		return fileinfo

	def missingfiles(tstfps,kind):
		missing = []
		for fp in tstfps:
			check = False
			for fname in fps:
				check = fnmatch(fname,fp) or check
			if not check:
				missing.append('Error: missing %s - %s'%(kind,fp))
		if not missing and kind == 'output':
			missing.append('Error: unknown')
		return missing

	if not type(i) is list: i = [i]
	if not type(o) is list: o = [o]
	errors = []
	status = 0

	# compare files to fingerprints
	for tstfps in [o,i]:

		# reset variables for i/o
		fps = {}
		content = {}

		for fp in tstfps:

			for fname in glob(fp):
				# read file system for file's fingerprint
				fps[fname] = getfingerprint(fname)
				# override getfingerprint() values with infodata values
				if 'start'  in infodata: fps[fname]['start']  = infodata['start']
				if 'finish' in infodata: fps[fname]['finish'] = infodata['finish']
				if 'misc'   in infodata: fps[fname]['misc']   = infodata['misc']

				# identify info file
				infoname = os.sep.join([os.path.dirname(fname),'info'])
				if not infoname in content: content[infoname] = {'info':[], 'fp':[]}
				if os.path.exists(infoname):
					# read stored fingerprint from info file
					errmsg = '''HALT: The current graph will use an %s file named:\n\n%s\n\nThis file exists and was fingerprinted on %s.\n\nSince then, the file was changed without updating the fingerprint.\n\nIf you want to use the changed file, delete the \"info\" file in the same folder and run this command again.\n\nIf you want to rebuild the file, delete or rename the existing file and run this command again.'''
					errmsg = errmsg%('input' if tstfps == i else 'output',fname,'%s')
					if not content[infoname]['info']:
						content[infoname]['info'] = getcontent(infoname)
						content[infoname]['fp'],error = parsefingerprint(fname,fps[fname],content[infoname]['info'],forceupdate,errmsg)
					else:
						content[infoname]['fp'],error = parsefingerprint(fname,fps[fname],content[infoname]['fp'],forceupdate,errmsg)
					errors.extend(error)
				else:
					# create info file contents the first time
					content[infoname]['info'] = []
					content[infoname]['fp'].extend(setfingerprint(fps[fname]))

		if not fps and tstfps == o:
			# output not created
			status = 1
		elif errors or len(fps) < len(tstfps) or (status == 1 and not i):
			# files incomplete
			errors.extend(missingfiles(tstfps,'input' if tstfps == i else 'output'))
			break
		else:
			# update the info file
			for infoname in content:
				if not sorted(content[infoname]['info']) == sorted(content[infoname]['fp']):
					setcontent(infoname,content[infoname]['fp'])
			status = 0 if tstfps == o else 1 if status == 1 else 0

	if errors:
		# return error list
		return {'errors': errors}
	elif status == 0:
		return {'bypass': 'files exist'}
	elif status == 1:
		# output files don't exist. return message to start
		return {'start':'%s'%(timestamp())}

def file_length(fname):
	with open(fname) as f:
		for i, l in enumerate(f):
			pass
	return i + 1

def samplepopulations(popsize,listsize,nbrlists=1):
	'''returns 'nbrlists' lists of random numbers with 'listsize' elements from 1 to 'popsize' '''
	import random
	if popsize < 3: return [[]]*nbrlists
	if listsize >= popsize: listsize = 1
	tmplist = random.sample(xrange(popsize),listsize*nbrlists)
	tmplen = int(len(tmplist)/nbrlists)
	i = 0
	return [tmplist[i:i+tmplen] for i in range(i,listsize*nbrlists,tmplen)]

def samplesize(popsize,confidencelevel,confidenceinterval,estimate=0.5):
	if not popsize: return 0
	if confidencelevel <= 0.0: return 0
	elif confidencelevel >= 1.0: return popsize
	if not confidenceinterval:
		confidenceinterval = 0.001
	elif int(confidenceinterval) == confidenceinterval:
		confidenceinterval = confidenceinterval/100.0
	ninetyfive=1.959964 # // 95% z-value
	ninetynine=2.575829 # // 99% z-value
	zval = ninetyfive if confidencelevel < 0.99 else ninetynine
#	zval = _zval(confidencelevel)
	std_err = (confidenceinterval*2)/(zval*2)
	n = (estimate*(1-estimate))/(std_err*std_err)
	f = (n)/(popsize + n - 1)
	n = n * (1-f)
	return int(round(n))

#def _zval(p):
#	import stats
#	zscoremax = 6
#	z_epsilon = 0.001	#Accuracy of z approximation
#	zscoremin = -zscoremax
#	zval = (zscoremax + zscoremin) * 0.5
#	pval = stats.zprob(zval)
#	while not p-z_epsilon<=pval<=p+z_epsilon:
#		if (pval >= p):
#			zscoremax = zval
#		else:
#			zscoremin = zval
#		zval = (zscoremax + zscoremin) * 0.5


#		pval = stats.zprob(zval)
#	return zval

buildset = {
	'f':'',
	'e':'',
	'tmbuildname':'',
	'alignertype':'',
	'tmgrams':'',
	'lmbuildname':'',
	'lmtype':'',
	'lmgrams':'',
	'name':'',
	}

def parseencodedfolder(fn):

	fn = fn.split('-s=',1)
	if len(fn) == 2:
		fn[1] = 's=%s'%(fn[1])
	else:
		fn = fn[0].split('-t=',1)
		if len(fn) == 2:
			fn[1] = 't=%s'%(fn[1])

	if len(fn) == 1:
		fn.append(dict(buildset))
	else:
		replace = (
			('s','f'),
			('t','e'),
			('p','tmbuildname'),
			('a','alignertype'),
			('g','tmgrams'),
			('l','lmbuildname'),
			('T','lmtype'),
			('n','lmgrams'),
			('c','tmbuildname":"%(lmbuildname)s","lmbuildname'),
			('z','tmgrams":"%(lmgrams)s","lmgrams'),
			)
		for i,j in replace:
			fn[1] = fn[1].replace('-%s='%(i),'\",\"%s='%(j))
		fn[1] = fn[1].replace('=','\":\"')
		fn[1] = eval('{\"%s\"}'%(fn[1]))
		if fn[0] == 'recaser':
			if 'lmbuildname' in fn[1] and 'tmbuildname' in fn[1]:
				fn[1]['tmbuildname'] = fn[1]['tmbuildname']%(fn[1])
			if 'lmgrams' in fn[1] and 'tmgrams' in fn[1]:
				fn[1]['tmgrams'] = fn[1]['tmgrams']%(fn[1])
	fn[1]['name'] = fn[0]
	return fn[1]

lmbytype = {
	'srilm': '0',
	'irstlm': '1',
	'randlm': '5',
	'srilmken': '9',
	'irstlmken': '9',
	}
lmbycode = {
	'0': 'srilm',
	'1': 'irstlm',
	'5': 'randlm',
	'9': 'irstlmken',
#	'9': 'srilmken',
	}

def parse_ttable_files(text,module,binary=True,ttable_files=[],cfgfile=''):
	errors = []
	for item in [item.strip() for item in text if item.strip()]:
		item = item.split(None,4)
		if len(item) == 5:
			if isinteger(item[0]) and isinteger(item[1]) and isinteger(item[2]) and isinteger(item[3]) and os.sep in item[4]:
				if binary and os.path.splitext(item[4])[1] == '.gz':
					item[0] = '1'
					item[4] = os.path.splitext(item[4])[0]
				if not binary and not os.path.splitext(item[4])[1] == '.gz':
					item[0] = '0'
					item[4] = '%s.gz'%(item[4])
				if not ' '.join(item) in ttable_files: ttable_files.append(' '.join(item))
			else:
				errors.append([module,'invalid','\"[ttable-file] %s\" invalid%s.'%(' '.join(item),' in %s'%(cfgfile) if cfgfile else '')])
		else:
			errors.append([module,'invalid','\"[ttable-file] %s\" invalid%s.'%(' '.join(item),' in %s'%(cfgfile) if cfgfile else '')])
	if not ttable_files:
		errors.append([module,'invalid','\"[ttable-file]\" invalid%s.'%(' in %s'%(cfgfile) if cfgfile else '')])
	return ttable_files,errors

def parse_distortion_files(text,module,binary=True,distortion_files=[],cfgfile=''):
	if type(text) is str or type(text) is unicode:
		text = text.split(',')
	errors = []
	for item in [item.strip() for item in text if item.strip()]:
		item = item.split(None,3)
		if len(item) == 4:
			tstreordering = True
			for aspect in item[1].split('-'):
				tstreordering = aspect in ['wbe','phrase','hier','mslr','msd','monotonicity','leftright','backward','forward','bidirectional','fe','f','allff','collapseff'] and tstreordering
			if isinteger(item[0].replace('-','').replace('+','').replace(',','')) and tstreordering and isinteger(item[2]) and os.sep in item[3]:
				if binary and os.path.splitext(item[3])[1] == '.gz':
					item[3] = os.path.splitext(item[3])[0]
				if not binary and not os.path.splitext(item[3])[1] == '.gz':
					item[3] = '%s.gz'%(item[3])
				if not ' '.join(item) in distortion_files: distortion_files.append(' '.join(item))
			else:
				errors.append([module,'invalid','\"[distortion-file] %s\" invalid%s.'%(' '.join(item),' in %s'%(cfgfile) if cfgfile else '')])
		else:
			errors.append([module,'invalid','\"[distortion-file] %s\" invalid%s.'%(' '.join(item),' in %s'%(cfgfile) if cfgfile else '')])
	if not distortion_files:
		errors.append([module,'invalid','[distortion-file] invalid%s.'%(' in %s'%(cfgfile) if cfgfile else '')])
	return distortion_files,errors

def parse_generation_files(text,module,generation_files=[],cfgfile=''):
	if type(text) is str or type(text) is unicode:
		text = text.split(',')
	errors = []
	for item in [item.strip() for item in text if item.strip()]:
		item = item.split(None,3) # 0 2 2 /.../pos-decomposed/model/generation.0-2
		if len(item) == 4:
			if isinteger(item[0]) and isinteger(item[1]) and isinteger(item[2]) and os.sep in item[3]:
				if not item in generation_files: generation_files.append(' '.join(item))
			else:
				errors.append([module,'invalid','\"[generation-file] %s\" invalid%s'%(' '.join(item),' in %s'%(cfgfile) if cfgfile else '')])
		else:
			errors.append([module,'invalid','\"[generation-file] %s\" invalid%s'%(' '.join(item),' in %s'%(cfgfile) if cfgfile else '')])
	if not generation_files:
		errors.append([module,'invalid','[generation-file] invalid%s.'%(' in %s'%(cfgfile) if cfgfile else '')])
	return generation_files,errors

def parse_lmodel_files(text,module,lmodel_files=[],cfgfile=''):
	if type(text) is str or type(text) is unicode:
		text = text.split(',')
	errors = []
	for item in [item.split(':',3) if ':' in item else item.split(None,3) for item in [item.strip() for item in text] if item.strip() and not item.lower().strip().replace(' ','') == '000/tmp/placeholder.lm' and not item.lower() == '0:0:/tmp/placeholder.lm:0']:
		if len(item) == 4:
			if os.sep in item[3]:						# moses.ini format (0 0 0 /tmp/placeholder.lm)
				lmcode,lmfactor,lmgrams,lmodel_file = item
			elif os.sep in item[2]:						# train-model.perl format (0:0:/tmp/placeholder.lm:0)
				lmfactor,lmgrams,lmodel_file,lmcode = item
			if lmcode in lmbycode.keys() and isinteger(lmfactor) and isinteger(lmgrams) and int(lmgrams) > 0 and os.sep in lmodel_file:
				if not ' '.join([lmcode,lmfactor,lmgrams,lmodel_file]) in lmodel_files: lmodel_files.append(' '.join([lmcode,lmfactor,lmgrams,lmodel_file]))
			else:
				errors.append([module,'invalid','\"[lmodel-file] %s\" invalid%s. Valid format: \"lmcode lmfactor lmgrams /path/filename\" where lmgrams > 0'%(' '.join(item),' in %s'%(cfgfile) if cfgfile else '')])
		else:
			errors.append([module,'invalid','[lmodel-file] \"%s\" invalid%s. Valid format: \"lmcode lmfactor lmgrams /path/filename\" where lmgrams > 0'%(' '.join(item),' in %s'%(cfgfile) if cfgfile else '')])
	if not lmodel_files:
		errors.append([module,'invalid','[lmodel-file] invalid%s.'%(' in %s'%(cfgfile) if cfgfile else '')])
	return lmodel_files,errors

def test_ttable_files(ttable_files):
	ttables = []
	success = True
	for item in [item.strip() for item in ttable_files if item.strip()]:
		item = item.split(None,4)
		if len(item) == 5:
			if os.path.splitext(item[4])[1] in ['.gz']:
				ttable = item[4]
				cnt = 1
			else:
				ttable = '%s.binphr.*'%(item[4])
				cnt = 5
			test = glob(ttable)
			if len(test) == cnt:
				for table in test:
					if not table in ttables: ttables.append(table)
			else:
				success = False
	if not ttables:
		success = False
	return success,ttables

def test_distortion_files(distortion_files):
	distortions = []
	success = True
	for item in [item.strip() for item in distortion_files if item.strip()]:
		item = item.split(None,3)
		if len(item) == 4:
			if os.path.splitext(item[3])[1] in ['.gz']:
				distortion = item[3]
				cnt = 1
			else:
				distortion = '%s.binlexr.*'%(item[3])
				cnt = 5
			test = glob(distortion)
			if len(test) == cnt:
				for table in test:
					if not table in distortions: distortions.append(table)
			else:
				success = False
		else:
			success = False
	if not distortions:
		success = False
	return success,distortions

def test_generation_files(generation_files):
	generations = []
	success = True
	for item in [item.strip() for item in generation_files if item.strip()]:
		item = item.split(None,3)
		if len(item) == 4:
			test = glob(item[3])
			if len(test) == 1:
				for table in test:
					if not table in generations: generations.append(table)
			else:
				success = False
		else:
			success = False
	if not generations:
		success = False
	return success,generations

def test_lmodel_files(lmodel_files):
	lmodels = []
	success = True
	for item in [item.split(':',3) if ':' in item else item.split(None,3) for item in [item.strip() for item in lmodel_files] if item.strip() and not item.strip().replace(' ','') == '000/tmp/placeholder.lm' and not item == '0:0:/tmp/placeholder.lm:0']:
		if len(item) == 4:
			if os.sep in item[3]:						# moses.ini format (0 0 0 /tmp/placeholder.lm)
				lmcode,lmfactor,lmgrams,lmodel_file = item
			elif os.sep in item[2]:						# train-model.perl format (0:0:/tmp/placeholder.lm:0)
				lmfactor,lmgrams,lmodel_file,lmcode = item
			test = glob(lmodel_file)
			if len(test) == 1:
				for model in test:
					if not model in lmodels: lmodels.append(model)
			else:
				success = False
		else:
			success = False
	if not lmodels:
		success = False
	return success,lmodels

## {{{ http://code.activestate.com/recipes/466302/ (r6+)
## 2011-12-20 adapted for CorpusFiltergraph: moved imports into functions, moved eval(key) into function, removed optparser code
def merge(chunks,key=None):

	from heapq import heapify, heappop, heappush

	if key is None:
		key = lambda x : x

	values = []

	for index, chunk in enumerate(chunks):
		try:
			iterator = iter(chunk)
			value = iterator.next()
		except StopIteration:
			try:
				chunk.close()
				os.remove(chunk.name)
				chunks.remove(chunk)
			except:
				pass
		else:
			heappush(values,((key(value),index,value,iterator,chunk)))

	while values:
		k, index, value, iterator, chunk = heappop(values)
		yield value
		try:
			value = iterator.next()
		except StopIteration:
			try:
				chunk.close()
				os.remove(chunk.name)
				chunks.remove(chunk)
			except:
				pass
		else:
			heappush(values,(key(value),index,value,iterator,chunk))

def batch_sort(inp,out,key=None,comp=None,buffer_size=32000,tempdirs=[]):

	from itertools import islice, cycle

	if type(key) is str or type(key) is unicode:
		key = eval('lambda line : (%s)'%key)

	if not tempdirs:
		from tempfile import gettempdir
		tempdirs = [gettempdir()]
	elif not type(tempdirs) is list:
		tempdirs = [tempdirs]

	input_file = file(inp,'rb',64*1024)
	try:
		input_iterator = iter(input_file)

		chunks = []
		try:
			for tempdir in cycle(tempdirs):
				current_chunk = list(islice(input_iterator,buffer_size))
				if current_chunk:
					current_chunk.sort(key=key,cmp=comp)
					output_chunk = file(os.path.join(tempdir,'%06i'%len(chunks)),'w+b',64*1024)
					output_chunk.writelines(current_chunk)
					output_chunk.flush()
					output_chunk.seek(0)
					chunks.append(output_chunk)
				else:
					break
		except:
			for chunk in chunks:
				try:
					chunk.close()
					os.remove(chunk.name)
				except:
					pass
			if output_chunk not in chunks:
				try:
					output_chunk.close()
					os.remove(output_chunk.name)
				except:
					pass
			return
	finally:
		input_file.close()

	output_file = file(out,'wb',64*1024)
	try:
		output_file.writelines(merge(chunks,key))
	finally:
		for chunk in chunks:
			try:
				chunk.close()
				os.remove(chunk.name)
			except:
				pass
		output_file.close()

#		'buffer_size','''Size of the line buffer. The file to sort is
#			divided into chunks of that many lines.
#			Default: 32,000 lines.'''

#		'key','''Python expression used to compute the key for each
#			line, \"lambda line:\" is prepended.
#			Example: -k \"line[5:10]\". Default: the whole line is
#			the key.'''

#		'tempdirs','''Temporary directory to use. You might get performance
#			improvements if the temporary directory is not on the
#			same physical disk than the input and output
#			directories. You can even try providing multiples
#			directories on differents physical disks. Use
#			multiple -t options to do that.'''

#	batch_sort(input,output,key,buffer_size,tempdirs)
## end of http://code.activestate.com/recipes/466302/ }}}

def getinstalledmodels():
	configpath = os.sep.join(['','usr','local','etc','domy','*.conf'])
	return [os.path.splitext(os.path.basename(model))[0] for model in glob(configpath)]

def chmodrecurse(path,uid,gid):
	for root, dirs, files in os.walk(path):
		for momo in dirs:
			os.chown(os.path.join(root, momo), uid, gid)
		for momo in files:
			os.chown(os.path.join(root, momo), uid, gid)

def levels2lists(levels):
	'''Create lists for 'levels' definition'''
	import itertools
	return dedupelist([list(x) for x in itertools.product(*levels)])

def str2datetime(text):
	import datetime
	return datetime.datetime.strptime(text[:-5], '%Y%m%dT%H%M%S')

def timestamp():
	import datetime
	dtnow,dtutcnow = datetime.datetime.now(), datetime.datetime.utcnow()
	delta = dtnow - dtutcnow
	datetime.timedelta(0,36000)
	hh,mm = divmod((delta.days * 24*60*60 + delta.seconds + 30) // 60, 60)
	return "%s%+2.2d:%2.2d"%(dtutcnow.isoformat().replace('T',' '), hh, mm)

def cksum(filename):
	return int(subprocess.Popen(['/usr/bin/cksum',filename],stdout=subprocess.PIPE).communicate()[0].split()[0])

def crc(filename):
	import zlib
	crc = 0
	try:
		for line in open(filename,'rb'):
			crc = zlib.crc32(line, crc)
	except:
		pass
#	print "crc-dec=%s"%crc
#	print "crc-hex=%X"%(crc & 0xFFFFFFFF)
	return crc

def getfreeram():
	return int(subprocess.Popen(['/usr/bin/free','-b'],stdout=subprocess.PIPE).communicate()[0].split('\n')[1].split()[1])

def cpu_count():
	from multiprocessing import cpu_count
	return cpu_count()

def setkwtypes(kw):
	for k in kw.keys():
		if k == 'bufsize':
			kw[k] = int(kw[k])
		elif k == 'executable':
			if not zbool(kw[k]): kw[k] = None; continue
			if os.name == 'nt' and not os.path.splitext(kw[k])[1] == '.exe': kw[k] = kw[k] + '.exe'
			path = whereis(kw[k])
			kw[k] = path if path else kw[k]
		elif k in ['stdin', 'stdout', 'stderr']:
			kw[k] = None if not zbool(kw[k]) else kw[k]
		elif k == 'preexec_fn':
			kw[k] = None if not zbool(kw[k]) or not os.name == 'posix' else kw[k]
		elif k == 'close_fds':
			kw[k] = zbool(kw[k])
		elif k == 'shell':
			kw[k] = zbool(kw[k])
		elif k == 'cwd':
			kw[k] = None if not zbool(kw[k]) else str(kw[k]).replace('\\',os.sep).replace('/',os.sep)
		elif k == 'env':
			kw[k] = None if not zbool(kw[k]) else dict(os.environ,**dict([pair.strip(' :').split('=') for pair in str(kw[k]).strip(',').split(',') if '=' in pair]))
		elif k == 'universal_newlines':
			kw[k] = zbool(kw[k])
		elif k == 'startupinfo':
			kw[k] = None if not zbool(kw[k]) or not os.name == 'nt' else kw[k]
		elif k == 'creationflags':
			kw[k] = int(kw[k]) if os.name == 'nt' else 0

def whereis(program,envSep = ':'):
	if os.path.isfile(os.path.abspath(os.path.expanduser(program.replace('/',os.sep).replace('\\',os.sep)))): return os.path.abspath(os.path.expanduser(program.replace('/',os.sep).replace('\\',os.sep)))
	tmpprog = os.path.basename(program)
	if os.name == 'nt': envSep = ';'
	for path in sys.path+os.environ.get('PATH','').split(envSep):
		if os.path.exists(os.sep.join([path,tmpprog])) and not os.path.isdir(os.sep.join([path,tmpprog])):
			return os.path.abspath(os.sep.join([path,tmpprog]))
	return ''

def tempname(filelist,prefix='~',extension='.tmp'):
	'''create file name in temp folder'''
	return os.sep.join([tempfile.gettempdir(),''.join([prefix,'`'.join([item.replace(':','_').replace('\\','`').replace('/','`') for item in filelist]),extension])])

def dedupelist(l):
	u = []
	for i in l:
		if not i in u: u.append(i)
	return u

def rreplace(s,old,new,count=None):
	count = count if count else len(s)
	li = s.rsplit(old,count)
	return new.join(li)

def isinteger(s):
	try:
		int(s)
		return True
	except:
		return False

def isnumeric(s):
	try:
		float(s)
		return True
	except:
		return False

def zbool(s):
	if str(s).lower() in ['false','f','n','0','']:
		return False
	elif str(s).lower() in ['true','t','y','1']:
		return True
	elif str(s).lower() in ['none']:
		return None
	elif isinteger(s):
		return int(s)
	elif isnumeric(s):
		return float(s)
	else:
		return s

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import parser\n\tfrom %s import manager\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0],
	os.path.splitext(os.path.basename(sys.argv[0]))[0],
	)

licensetxt=u'''CorpusFiltergraph™ v4.0
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
