@echo off

cd %~dp0..\..
set CORPORA=%CD%\CORPORA
set GRAPHS=%CD%\corpusfg\graphs
cd %~dp0..
set DOMY=%CD%

if exist %DOMY%\bin\corpusfg.py goto script
echo Error: Please verify installation.
goto :EOF

:script
set cfgm=%DOMY%\bin\corpusfg.py
goto :continue

:continue
cls


echo Introduction:
echo.
echo This is the first of two demos. This demo runs eight (8) example
echo "GRAPHS". GRAPHS are parallel tool-chains that extract, clean,
echo and align parallel text data. These example GRAPHS prepare text
echo text data for SMT training.
echo.
echo Each example displays a description, the GRAPH's command line, a
echo review of where to find the results and an introduction to the
echo next example.
echo.
echo You can find the GRAPHs and their config.ini files in:
echo %GRAPHS%
echo.


echo Next: Example 1 converts HTML/XML entities
pause
cls
echo Example 1: clean HTML/XML entities
echo.
echo Cleaning HTML/XML data is an important task for all data preparation.
echo HTML and XML documents often include "entities" that represent characters.
echo For example, "&lt;" is "<" and "&gt;" is ">" and "&aacute;" is "á".
echo This GRAPH converts these "entities" back to
echo "normal" characters.
echo.
echo Command:  $ python %cfgm% clean-lm
echo.
pause
cls
echo Example 1:  clean HTML/XML entities
echo.
echo Command:  $ python %cfgm% clean-lm
echo.
python %cfgm% clean-lm
echo.
echo Review: the results were saved in the output file:
echo   "%CORPORA%/DEMO.qc/Examples/Unicode/.../character-entities.txt"
echo.


echo Next: Example 2 extracts bilingual dictionaries from individual files
pause
cls
echo Example 2: extract dictionaries
echo.
echo Many documents have parenthetical in-line text that can be extracted and
echo converted to dictionaries. In this example, the zh_hk-en bitext input
echo files have this data in the zh_hk and en files. This GRAPH removes the
echo parenthetical entries from the original files and saves them as new
echo parallel bitext files.
echo.
echo Command:  $ python %cfgm% extract-dictionary
echo.
pause
cls
echo Example 2: extract dictionaries
echo.
echo Command:  $ python %cfgm% extract-dictionary
echo.
python %cfgm% extract-dictionary
echo.
echo Review: the results were saved in output files:
echo   "%CORPORA%/DEMO.qc/Examples/Legislation/statutes/..."
echo.


echo Next: Example 3 align sentences
pause
cls
echo Example 3: create sentence-aligned data from aligned files
echo.
echo This GRAPH starts with file-aligned data (fa) with one sentence
echo or phrase per line. The GRAPH matches source and target language
echo sentences to the (tm) output. Sentences without a match are saved
echo to the language model (lm) output.
echo.
echo Command:  $ python %cfgm% sa-champollion
echo.
pause
cls
echo Example 3: create sentence-aligned data from aligned files
echo.
echo Command:  $ python %cfgm% sa-champollion
echo.
copy %GRAPHS%/sa-champollion/config.win.ini %GRAPHS%/sa-champollion/config.ini
python %cfgm% sa-champollion
echo.
echo Review: the results were saved in output files:
echo   "%CORPORA%/DEMO.sa/Examples/..."
echo.


echo Next: Example 4 imports data from TMX
pause
cls
echo Example 4: import data TMX files
echo.
echo TMX (translation memory exchange) files were developed to exchange
echo translation memory data between companies, colleagues and systems.
echo This GRAPH imports tmx data and maintains alignment in the
echo CorpusFiltergraph folder hierarchy. These sample TMX files come from
echo the DGT-TM version 1 and include sentences across 13 languages.
echo.
echo Command:  $ python %cfgm% import-tmx
echo.
pause
cls
echo Example 4: import data TMX files
echo.
echo Command:  $ python %cfgm% import-tmx
echo.
python %cfgm% import-tmx
echo.
echo Review: the results were saved in output files:
echo   "%CORPORA%/DEMO.sa/European Commission/DGT_TM_1.0/..."
echo.


echo Next: Example 5 exports data to TMX files
pause
cls
echo Example 5: export data to TMX file
echo.
echo This GRAPH exports tm data to TMX files. Data from the CorpusFiltergarph
echo folder hierarchy can be shared with other translation systems.
echo.
echo Command:  $ python %cfgm% export-tmx
echo.
pause
cls
echo Example 5: export data to TMX file
echo.
echo Command:  $ python %cfgm% export-tmx
echo.
python %cfgm% export-tmx
echo.
echo Review: the results were saved in output files:
echo   "%CORPORA%/DEMO.raw/European Commission/Legislation/..."
echo.


echo Next: Example 6 normalizes Japanese text
pause
cls
echo Example 6: normalize Japanese text
echo.
echo Japanese text include four character sets: Kanji (Chinese characters),
echo Hiragana, Katakana and Romanji. Each set can have several variations
echo such as full-width and half-width. The Unicode normalization filter
echo converts these variations to one "composed" version.
echo.
echo Command:  $ python %cfgm% normalize-unicode
echo.
pause
cls
echo Example 6: normalize Japanese text
echo.
echo Command:  $ python %cfgm% normalize-unicode
echo.
python %cfgm% normalize-unicode
echo.
echo Review: the results were saved in output files:
echo   "$DOMY_DATA/CORPORA/DEMO.qc/Examples/Education/..."
echo.


echo Next: Example 7 cleans 200 pairs of files."
pause
cls
echo Example 7: prepare nl-en sample data (micro set)
echo.
echo This GRAPH prepares sentence-aligned (sa) text files for use as SMT
echo training corpus. Files in "tm" trees are processed as pairs. This
echo GRAPH's config.ini file defines:
echo      mintoken = 3
echo      maxtoken = 20
echo      ratio = 4:1
echo Sentence pairs within the values are saved to the
echo      "DEMO.ready_micro" output as for use as SMT training data.
echo Sentence pairs outside the values are saved to the
echo      "DEMO.ready_micro-workbench" output for review by qualified editors.
echo.
echo Command:  $ python %cfgm% clean-tm
echo.
pause
cls
echo Example 7: prepare nl-en sample data (micro set)
echo.
echo Command:  $ python %cfgm% clean-tm
echo.
ren %GRAPHS%/clean-tm/config.ini %GRAPHS%/clean-tm/config.ini.bak
copy %GRAPHS%/clean-tm/config.micro.ini %GRAPHS%/clean-tm/config.ini
python %cfgm% clean-tm
copy %GRAPHS%/clean-tm/config.ini.bak %GRAPHS%/clean-tm/config.ini
del %GRAPHS%/clean-tm/config.ini.bak
echo.
echo Review: the results were saved in output files:
echo   "%CORPORA%/DEMO.ready_micro/..." and
echo   "%CORPORA%/DEMO.ready_micro-workbench/..."
echo.


echo Next: Example 8 cleans the same files with different settings.
pause
cls
echo Example 8: prepare nl-en sample data (larger set)
echo.
echo This GRAPH starts with the same data as Example 7. New "mintoken",
echo and "maxtoken" settings create a larger SMT training data set. This
echo GRAPH's config file defines:
echo      mintoken = 2
echo      maxtoken = 65
echo      ratio = 4:1
echo Sentence pairs within the values are saved to the
echo      "DEMO.ready" output as for use as SMT training data.
echo Sentence pairs outside the values are saved to the
echo      "DEMO.ready-workbench" output for review by qualified editors.
echo.
echo Command:  $ python %cfgm% clean-tm
echo.
pause
cls
echo Example 8: prepare nl-en sample data (larger set)
echo.
echo Command:  $ python %cfgm% clean-tm
echo.
python %cfgm% clean-tm
echo.
echo Review: the results were saved in output files:
echo   "%CORPORA%/DEMO.ready/..." and
echo   "%CORPORA%/DEMO.ready-workbench/..."
echo.

echo Next: Review
pause
cls
echo Review:
echo.
echo Examples in Demo-1.sh extracted, cleaned and aligned parallel text data.
echo.
echo Example 1 converted HTML entities to UTF-8 characters
echo Example 2 extracted bilingual data from individual files
echo Example 3 created sentence-aligned files from file-aligned data
echo Example 4 imported TMX data for CorpusFiltergraph processing
echo Example 5 exported CorpusFiltergraph processed data to TMX files
echo Example 6 normalized Unicode data using Unicode specifications
echo Example 7 cleaned micro set of data for use as SMT training corpus
echo Example 8 cleaned the same SMT training data with different settings
echo.
echo NOTE: All outputs were saved in "%CORPORA%/DEMO.*"
echo   sub-folders. The \"demo-2.sh\" examples use these these outputs.
echo   It is safe to delete these sub-folders after running "demo-2.sh".
echo   You can re-run demo-1.sh and watch the demo create output files.
echo.
echo Demo-2.sh will prepare BUILD sets for SMT training from the cleaned data
echo and train two translation engines. You can start next demo with the
echo command "demo-2.sh"
echo.

echo Next: Repeat demo-1.sh or run demo-2.sh
rem pause
rem cls

rem CorpusFiltergraph™ v3.5.245
rem Copyright © 2010-2011 Precision Translation Tools Co., Ltd.

rem This program is free software: you can redistribute it and\or modify
rem it under the terms of the GNU Lesser General Public License as published by
rem the Free Software Foundation, either version 3 of the License, or
rem (at your option) any later version.

rem This program is distributed in the hope that it will be useful,
rem but WITHOUT ANY WARRANTY; without even the implied warranty of
rem MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
rem GNU Lesser General Public License for more details.

rem You should have received a copy of the GNU Lesser General Public License
rem along with this program.  If not, see http:\\www.gnu.org\licenses\

rem For more information, please contact Precision Translation Tools Co., Ltd.
rem at: http:\\www.precisiontranslationtools.com
