#! /bin/bash

DOMY=${0%/*}/..
[ -L "${0}" ] && DOMY=`readlink ${0}`
pushd ${DOMY%/*}/.. >/dev/null; export DOMY=`pwd -P`; popd >/dev/null
cfgm=`which domy-ce`
BASE=`basename $cfgm`
if [ ! -x "$cfgm" ] ; then
	echo "* $(date +%x\ %r) - ${0##*/} - installation corrupted - reinstall required"
	exit 1
	fi
"$cfgm" -vS
if [ $? -ne 0 ] ; then echo "* $(date +%x\ %r) - ${0##*/} - configuration failure - reinstallation required" ; exit 1 ; fi
if [ ! -e "$HOME/domy/$BASE.ini" ] ; then echo "* $(date +%x\ %r) - ${0##*/} - configuration failure - reinstallation required" ; exit 1 ; fi
while read line ; do
	if [[ "${line}" =~ "=" ]] ; then eval "$line" ; fi
	done < "$HOME/domy/$BASE.ini"
args=($*)

usage(){
	cat <<- HELP
	Usage: ${0##*/} [OPTION...]
	Run the first demo wizard
	
	Options:
	  -c       continue without pausing to read prompts
	  -h       this help text
	
	Examples:
	  time ${0##*/} -c
	     This command is useful to measure end-to-end performance.
	
	HELP
	exit $1
	}

[[ "${args[@]}" =~ "-h" ]] && usage 0
[[ "${args[@]}" =~ "--help" ]] && usage 0

clear

echo "Introduction:"
echo
echo "This is the first of two demos. This demo runs five (5) example"
echo "\"GRAPHS\". GRAPHS are parallel tool-chains that extract, clean,"
echo "align parallel text, and otherwise prepare corpora and translation"
echo "memory data for use as SMT training data."
echo
echo "Example 1 converts HTML entities to UTF-8 characters"
echo "Example 2 creates sentence-aligned files from file-aligned data"
echo "Example 3 imports data from TMX files to CorpusFiltergraph"
echo "Example 4 exports data from CorpusFiltergraph to TMX files"
echo "Example 5 extracts bilingual data from individual files"
echo
echo "You can find the GRAPHs and their config.ini files in:"
echo "$graphs"
echo

echo "Next: Instructions"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Instructions:"
echo
echo "This is a wizar-style demo. It pauses before running an example"
echo "with a short description and the GRAPH's command line."
echo
echo "It pauses after running the example to show the command line,"
echo "the input and output of the example. Right-click the input or"
echo "output path and select \"Open Link\" to see the changes."
echo

echo "Next: Example 1, clean HTML/XML entities"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Example 1: clean HTML/XML entities"
echo
echo "Cleaning HTML/XML data is an important task for all data preparation."
echo "HTML and XML documents often include \"entities\" that represent characters."
echo "For example, &lt; is \"<\" and &gt; is \">\" and &aacute; is \"á\". This GRAPH"
echo "converts these \"entities\" back to \"normal\" characters."
echo
echo "Command:  ~\$ $BASE clean-lm"

echo
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 1..."
clear

echo "Example 1: clean HTML/XML entities"
echo
if [ -f "$graphs/clean-lm/config.demo.ini" ]; then
	cp $graphs/clean-lm/config.ini $graphs/clean-lm/.config.ini.tmp
	cp $graphs/clean-lm/config.demo.ini $graphs/clean-lm/config.ini
	python $cfgm -v clean-lm
	result=$?
	mv $graphs/clean-lm/.config.ini.tmp $graphs/clean-lm/config.ini
	[ $result -ne 0 ] && exit 1
	echo
	echo "Review"
	echo "Command:  ~\$ $BASE clean-lm"
	echo "Input:    \"file://$rootfolder/CORPORA/sa/Examples/Unicode/entities/lm/en/en/character-entities.txt\""
	echo "Output:   \"file://$rootfolder/CORPORA.demo/qc/Examples/Unicode/entities/lm/en/en/character-entities.txt\""
	echo
else
	echo "\"clean-lm\" demo configuration is not installed."
	fi

echo "Next: Example 2, create sentence-aligned data from aligned files"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Example 2: create sentence-aligned data from aligned files"
echo
echo "This GRAPH starts with file-aligned data (fa) with one sentence"
echo "or phrase per line. The GRAPH matches source and target language"
echo "sentences to the (tm) output. Sentences without a match are saved"
echo "to the language model (lm) output."
echo
echo "Command:  ~\$ $BASE sa-champollion"
echo
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 2..."
clear

echo "Example 2: create sentence-aligned data from aligned files"
echo
echo "Command:  ~\$ $BASE sa-champollion"
echo
if [ -f "$graphs/sa-champollion/config.demo.ini" ]; then
	cp $graphs/sa-champollion/config.ini $graphs/sa-champollion/.config.ini.tmp
	cp $graphs/sa-champollion/config.demo.ini $graphs/sa-champollion/config.ini
	python $cfgm -v sa-champollion
	result=$?
	mv $graphs/sa-champollion/.config.ini.tmp $graphs/sa-champollion/config.ini
	[ $result -ne 0 ] && exit 1
	echo
	echo "Review"
	echo "Command:  ~\$ $BASE sa-champollion"
	echo "Input1:   \"file://$rootfolder/CORPORA/fa/Examples/News/general/tm/zh_cn/zh_cn/CPP20000210000021.txt\""
	echo "Input2:   \"file://$rootfolder/CORPORA/fa/Examples/News/general/tm/zh_cn/en/CPP20000210000021.txt\""
	echo "Output1:  \"file://$rootfolder/CORPORA.demo/sa/Examples/News/general/tm/zh_cn/zh_cn/CPP20000210000021.txt\""
	echo "Output2:  \"file://$rootfolder/CORPORA.demo/sa/Examples/News/general/tm/zh_cn/en/CPP20000210000021.txt\""
else
	echo "\"sa-champollion\" demo configuration is not installed."
	fi

echo "Next: Example 3, import data from TMX files"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Example 3: import data TMX files"
echo
echo "TMX (translation memory exchange) files were developed to exchange"
echo "translation memory data between companies, colleagues and systems."
echo "This GRAPH imports aligned tmx data into the CorpusFiltergraph"
echo "folder hierarchy. These sample TMX files come from the DGT-TM"
echo "version 1 and maintain sentence alignment across 13 languages."
echo
echo "Command:  ~\$ $BASE import-tmx"
echo
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 3..."
clear

echo "Example 3: import data TMX files"
echo
echo "Command:  ~\$ $BASE import-tmx"
echo
if [ -f "$graphs/import-tmx/config.demo.ini" ]; then
	cp $graphs/import-tmx/config.ini $graphs/import-tmx/.config.ini.tmp
	cp $graphs/import-tmx/config.demo.ini $graphs/import-tmx/config.ini
	python $cfgm -v import-tmx
	result=$?
	mv $graphs/import-tmx/.config.ini.tmp $graphs/import-tmx/config.ini
	[ $result -ne 0 ] && exit 1
	echo
	echo "Review"
	echo "Command:  ~\$ $BASE import-tmx"
	echo "Input1:   \"file://$rootfolder/RAW/European%20Commission/DGT_TM_1.0/Volume_1/21985A0705(01).tmx\""
	echo "Input2:   \"file://$rootfolder/RAW/European%20Commission/DGT_TM_1.0/Volume_1/22001D0118(18).tmx\""
	echo "Output1:  \"file://$rootfolder/CORPORA.demo/sa/European%20Commission/DGT_TM_1.0/Volume_1/tm/en_gb/\""
	echo
else
	echo "\"import-tmx\" demo configuration is not installed."
	fi

echo "Next: Example 4, export data to TMX files"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Example 4: export data to TMX files"
echo
echo "This GRAPH exports tm data to TMX files. Data from the CorpusFiltergarph"
echo "folder hierarchy can be shared with other translation systems."
echo
echo "Command:  ~\$ $BASE export-tmx"
echo
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 4..."
clear

echo "Example 4: export data to TMX files"
echo
echo "Command:  ~\$ $BASE export-tmx"
echo
if [ -f "$graphs/export-tmx/config.demo.ini" ]; then
	cp $graphs/export-tmx/config.ini $graphs/export-tmx/.config.ini.tmp
	cp $graphs/export-tmx/config.demo.ini $graphs/export-tmx/config.ini
	python $cfgm -v export-tmx
	result=$?
	mv $graphs/export-tmx/.config.ini.tmp $graphs/export-tmx/config.ini
	[ $result -ne 0 ] && exit 1
	echo
	echo "Review"
	echo "Command:  ~\$ $BASE export-tmx"
	echo "Inputs:   \"file://$rootfolder/CORPORA.demo/sa/European%20Commission/DGT_TM_1.0/Volume_1/tm/en_gb/\""
	echo "Output1:  \"file://$rootfolder/RAW.demo/European%20Commission/DGT_TM_1.0/Volume_1/tm/en_gb/21985A0705(01).tmx.txt.tmx\""
	echo "Output2:  \"file://$rootfolder/RAW.demo/European%20Commission/DGT_TM_1.0/Volume_1/tm/en_gb/22001D0118(18).tmx.txt.tmx\""
	echo
else
	echo "\"export-tmx\" demo configuration is not installed."
	fi

echo "Next: Example 5, extract bilingual dictionaries from individual files"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Example 5: extract bilingual dictionaries from individual files"
echo
echo "This is an example of a custom GRAPH. Users and PTTools can create"
echo "custom GRAPHS for special data-cleaning purposes. This example's"
echo "input file has parenthetical entries in-line with the main text."
echo "This GRAPH removes the parenthetical entries from the original files"
echo "and saves them as new parallel dictionary files."
echo
echo "Command:  ~\$ $BASE extract-dictionary"
echo
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 5..."
clear

echo "Example 5: extract bilingual dictionaries from individual files"
echo
echo "Command:  ~\$ $BASE extract-dictionary"
echo
if [ -f "$graphs/extract-dictionary/config.demo.ini" ]; then
	cp $graphs/extract-dictionary/config.ini $graphs/extract-dictionary/.config.ini.tmp
	cp $graphs/extract-dictionary/config.demo.ini $graphs/extract-dictionary/config.ini
	python $cfgm -v extract-dictionary
	result=$?
	mv $graphs/extract-dictionary/.config.ini.tmp $graphs/extract-dictionary/config.ini
	[ $result -ne 0 ] && exit 1
	echo
	echo "Review"
	echo "Command:  ~\$ $BASE extract-dictionary"
	echo "Input1:   \"file://$rootfolder/CORPORA/sa/Examples/Legislation/statutes/tm/zh_hk/zh_hk/hklaws.txt\""
	echo "Input2:   \"file://$rootfolder/CORPORA/sa/Examples/Legislation/statutes/tm/zh_hk/en/hklaws.txt\""
	echo "Outputs:  \"file://$rootfolder/CORPORA.demo/qc/Examples/Legislation/statutes/\""
	echo "Output1:  \"file://$rootfolder/CORPORA.demo/qc/Examples/Legislation/statutes/tm/zh_hk/zh_hk/hklaws.txt\""
	echo "Output2:  \"file://$rootfolder/CORPORA.demo/qc/Examples/Legislation/statutes/tm/zh_hk/en/hklaws.txt\""
	echo "Output3:  \"file://$rootfolder/CORPORA.demo/qc/Examples/Legislation/statutes/tm-dictionary/zh_hk/zh_hk/hklaws.txt\""
	echo "Output4:  \"file://$rootfolder/CORPORA.demo/qc/Examples/Legislation/statutes/tm-dictionary/zh_hk/en/hklaws.txt\""
	echo
else
	echo "\"extract-dictionary\" demo configuration is not installed."
	fi

echo "Next: Review"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Review:"
echo
echo "Examples in Demo-1.sh extracted, cleaned and aligned parallel text data."
echo
echo "Example 1 converted HTML entities to UTF-8 characters"
echo "Example 2 created sentence-aligned files from file-aligned data"
echo "Example 3 imported data from TMX files to CorpusFiltergraph"
echo "Example 4 exported data from CorpusFiltergraph to TMX files"
echo "Example 5 extracted bilingual data from individual files"
echo
echo "NOTE: All outputs were saved to sub-folders in"
echo "  \"$rootfolder/CORPORA.demo/*\" or \"$rootfolder/RAW.demo/*\"."
echo "  The examples in the next demo, \"demo-2.sh,\" continue with the"
echo "  outputs from this demo. It is safe to delete these demo root"
echo "  folders after running \"demo-2.sh\"."
echo
echo "Demo-2.sh prepares two data BUILD sets from the cleaned data and"
echo "trains two translation engines. You can start then next demo with"
echo "the command \"demo-2.sh\""
echo

echo "Next: Repeat demo-1.sh or run demo-2.sh"

#CorpusFiltergraph™ v4.0.264
#Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU Lesser General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Lesser General Public License for more details.

#You should have received a copy of the GNU Lesser General Public License
#along with this program.  If not, see http://www.gnu.org/licenses/.

#For more information, please contact Precision Translation Tools Co., Ltd.
#at: http://www.precisiontranslationtools.com
