#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#4.0.264 - updated to fully cover 'lm' keys

import os
import sys
import subprocess
import logging
import common as cf

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))

class aligner(object):

# ARGS key:
# -d dictf : use dictf as the translation dictionary
# -s xstop : use words in file xstop as X stop words
# -c n     : number of Y chars for each X char
# -n       : disallow 1-3, 3-1, 1-4, 4-1 alignments
# -f       : faster performance, lower accuracy
# -a       : align all and merge ommision
# -l       : use on-disk database to reduce memory usage

	cfg = {
		'args': {},
		'args1': '-c',
		'args2': '0.634',
		'args3': '-s',
		'args4': 'stoplist',
		'args5': '-d',
		'args6': 'dictionary',
		'exeoptions': '',
		'invertx2y': False,
		'kw': {
			'bufsize': 0,
			'executable': '4,perl',
			'stdin': 'PIPE',
			'stdout': 'PIPE',
			'stderr': 'PIPE',
			'preexec_fn': None,
			'close_fds': False,
			'shell': False,
			'cwd': None,
			'env': None,
			'universal_newlines': False,
			'startupinfo': None,
			'creationflags': 0,
			},
		'script': '2,bin%schampollion'%(os.sep),
		'encoding': 'utf8',
		'lowmem': False,
		'simple-only': False,
		'fast': False,
		'no-omissions': False,
		'version': '4.0.264',
		}
	args = {}
	encoding = 'utf8'
	invertx2y = False
	kw = {}
	script = '2,bin%schampollion'%(os.sep)
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		args = dict(cfg['args'])
		self.kw = dict(cfg['kw'])
		if cfg['script']:
			self.script = self.p.decodepath('%s,%s'%(__name__.split(',')[1],cfg['script']),True)
			if not self.script:
				self.errors.append([__name__,'missing',cfg['script']])
				logger.error('%s\t%s',*self.errors[-1][1:])
			args[0] = self.script
			self.kw['cwd'] = os.path.dirname(os.path.dirname(self.script))
			self.kw['env'] = dict(os.environ,**{'CTK': os.path.dirname(os.path.dirname(self.script))})
		else:
			self.script = None
		# convert args dict to args list
		args = [str(args[k]) for k in sorted(args.keys())]
		if 'executable' in cfg['kw'] and cfg['kw']['executable']:
			self.kw['executable'] = self.p.decodepath('%s,%s'%(__name__.split(',')[1],cfg['kw']['executable']),True)
			if not self.kw['executable']:
				self.errors.append([__name__,'missing',cfg['kw']['executable']])
				logger.error('%s\t%s',*self.errors[-1][1:])
			args.insert(0,cfg['exeoptions'] if cfg['exeoptions'] else '')
		else:
			self.kw['executable'] = None
		self.kw['stdin'] = subprocess.PIPE if cfg['kw']['stdin'] == 'PIPE' else cfg['kw']['stdin']
		self.kw['stdout'] = subprocess.PIPE if cfg['kw']['stdout'] == 'PIPE' else cfg['kw']['stdout']
		self.kw['stderr'] = subprocess.PIPE if cfg['kw']['stderr'] == 'PIPE' else cfg['kw']['stderr']
		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']

		self.invertx2y = cfg['invertx2y']
		self.resources = self.p.decodepath(','.join(__name__.split(',')[:3]+['3']+[__name__.split(',')[-1]]))
		if not os.path.exists(self.resources):
			self.errors.append([__name__,'missing',self.resources])
			logger.warn('%s\t%s',*self.errors[-1][1:])

		for i in range(0,len(args),2):
			self.args[0 if not args[i] else args[i]] = args[i+1]
		if cfg['lowmem']:
			self.args['-l'] = None
		if cfg['simple-only']:
			self.args['-n'] = None
		if cfg['fast']:
			self.args['-f'] = None
		if cfg['no-omissions']:
			self.args['-a'] = None

	def run(self,ktgt):
		'''run filter'''
		if not sum([len(line.strip()) for line in self.p.cfoutput[ktgt]['tempbuff']]): return
		ksrc = list(ktgt)
		ksrc[cf.rdrlang] = ktgt[cf.srclang]
		ksrc = tuple(ksrc)
		ktgtlm = list(ktgt)
		ktgtlm[cf.kind] = ktgtlm[cf.kind].replace(u'tm',u'lm',1)
		ktgtlm = tuple(ktgtlm)
		ksrclm = list(ksrc)
		ksrclm[cf.kind] = ksrclm[cf.kind].replace(u'tm',u'lm',1)
		ksrclm = tuple(ksrclm)

		# test if special-split.py module used property in the graph
		if not ksrc in self.p.cfextract or not ktgt in self.p.cfextract:
			logger.error('%s\t%s',*['missing','\"[%s]\" \"filterX=0,special-split\" not before \"filterY=2,%s'%(ktgt[cf.srclang],__name__)])
			return 249

		# add target lang extensions to dictionary names
		dargs = dict(self.args)

		# set keys and indexes for x2y or y2x
		if self.invertx2y:
			kx,kxlm = ktgt,ktgtlm
			ky,kylm = ksrc,ksrclm
			xlang,ylang = cf.rdrlang,cf.srclang
			dargs['-c'] = str(1-float(dargs['-c']))
		else:
			kx,kxlm = ksrc,ksrclm
			ky,kylm = ktgt,ktgtlm
			xlang,ylang = cf.srclang,cf.rdrlang

		if os.path.exists(os.sep.join([self.resources,os.extsep.join([dargs['-d'],'-'.join([ktgt[xlang],ktgt[ylang]])])])):
			dargs['-d'] = os.sep.join([self.resources,os.extsep.join([dargs['-d'],'-'.join([ktgt[xlang],ktgt[ylang]])])])
		else:
			del(dargs['-d'])
			dargs['-c'] = '0.500'
		if os.sep.join([self.resources,os.extsep.join([dargs['-s'],ktgt[xlang]])]):
			dargs['-s'] = os.sep.join([self.resources,os.extsep.join([dargs['-s'],ktgt[xlang]])])
		else:
			del(dargs['-s'])
		args = []
		for k in sorted(dargs.keys()):
			if dargs[k]:
				args.extend([k if k else '', dargs[k]])
			else:
				args.extend([k if k else '',])

		# add x,y inputs and output paths (note: '-' sends output to STDOUT)
#		falign = os.sep.join([self.p.tempdir,os.extsep.join([self.p.filelist[cf.basename],self.p.datetime.replace(':',''),'align'])])
		falign = '-'
		args.extend([self.p.cfoutput[kx]['tempname'],self.p.cfoutput[ky]['tempname'],falign])
		# save buffer to temp files for champollion input
		if self.p.writer.run(self.p.cfoutput): return -2

		# clear buffers for champollion re-alignment
		self.p.cfoutput[ktgt]['tempbuff'] = []
		self.p.cfoutput[ksrc]['tempbuff'] = []

		#run champollion script
		alignments = self.__filter(args)
		if not alignments:
			logger.warn('%s\t%s',*['config','no alignments found'])
			return -2

		# parse report results
		for alignment in alignments:
			xlines, ylines = [line.split(',') for line in alignment]
			xlines = [int(line)-1 for line in xlines]
			ylines = [int(line)-1 for line in ylines]
			# move matching line numbers from self.p.cfextract to self.p.cfoutput
			if xlines[0] == -1:
				yline = []
				for yitem in ylines:
					if yitem >= len(self.p.cfextract[ky]['tempbuff']): continue
					yline.append(self.p.cfextract[ky]['tempbuff'][yitem].strip())
				self.p.cfoutput[kylm]['tempbuff'].append(' '.join(yline))
			elif ylines[0] == -1:
				xline = []
				for xitem in xlines:
					if xitem >= len(self.p.cfextract[kx]['tempbuff']): continue
					xline.append(self.p.cfextract[kx]['tempbuff'][xitem].strip())
				self.p.cfoutput[kxlm]['tempbuff'].append(' '.join(xline))
			else:
				xline = []
				for xitem in xlines:
					xline.append(self.p.cfextract[kx]['tempbuff'][xitem].strip())
				yline = []
				for yitem in ylines:
					yline.append(self.p.cfextract[ky]['tempbuff'][yitem].strip())
				self.p.cfoutput[kx]['tempbuff'].append(' '.join(xline))
				self.p.cfoutput[ky]['tempbuff'].append(' '.join(yline))

	def flush(self,ktgt):
		return

	def close(self):
		return

	def __filter(self,args):
		'''run champollion'''
		try:
			subproc = subprocess.Popen(args,**self.kw)
		except Exception,e:
			raise RuntimeError("[%s] failed to start - %s"%(__name__,str(e),))
		for errline in iter(subproc.stderr.readline,''):
#			sys.stderr.write('%s\n'%(errline.rstrip()))
			pass
		if subproc.returncode:	# champollion returned error
			alignments = []
			if os.path.exists(args[-1]): os.unlink(args[-1])
		else:
			if args[-1] == '-':
				txtout = subproc.stdout.readlines()
			else:
				if not os.path.exists(args[-1]):
					alignments = []
				else:
					# open, read & close, delete alignment file
					txtout = open(args[-1],'r').read().strip().splitlines()
					os.unlink(args[-1])
			alignments = [alignment.replace('omitted','0').strip().split('<=>') for alignment in txtout]
		return alignments

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''CorpusFiltergraph™ v4.0
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
