#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#4.0.264 - version update

import sys
import os
from flock import flock
import codecs
from progress_bar import ProgressBar
import shutil
import logging
from filtergraph import filtergraph
import common as cf

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],__name__]))
realname = os.path.splitext(os.path.basename(os.path.realpath(sys.argv[0])))[0]

class manager(object):
	'''manager class processes batches of files through the filtergraph'''

	import socket
	cfg = {
		'cleanup': False,
		'command': 'run',
		'debug': False,
		'files': [],
		'hostname': socket.gethostname(),
		'markreadonly': False,
		'pid': os.getpid(),
		'resetreadonly': False,
		'restart': False,
		'runs': 1,
		'umask': 0,
		'usemanager': True,
		'verbose': logging.WARNING,
		'version': '4.0.264',
		'versions': False,
		}
	command = 'run'
	datetime = ''
	debug = False
	hostname = socket.gethostname()
	files = []
	filtergraph = filtergraph()
	lock = ''
	markreadonly = False
	pid = os.getpid()
	resetreadonly = False
	restart = False
	runs = 1
	umask = 0
	usemanager = True
	verbose = logging.WARNING
	srcqueue = ''
	isloaded = False
	isopen = False
	del(socket)

	def load(self,cfg):

		errors = []

		try:

			if self.isloaded:
				logger.warn('%s\t%s',*['skip','already loaded'])
				return errors

			# validate required defaults & merge configuration
			for k in self.cfg.keys():
				if k in cfg and cfg[k]: continue
				cfg[k] = self.cfg[k]
				if cfg[k] is None:
					errors.append([__name__,'missing','[%s] \"%s=\"'%(__name__,k)])
					logger.error('%s\t%s',*errors[-1][1:])
			_setDataTypes(self.cfg,cfg)

			os.umask(cfg['umask'])
			self.verbose = logging.DEBUG if cfg['debug'] else cfg['verbose'] if cfg['verbose'] else logging.WARNING

			# create file handler which logs messages
			self.logfile = 	os.path.abspath(os.path.expanduser(os.sep.join([cfg['filtergraph']['graphs'],cfg['filtergraph']['graphname'],os.extsep.join([cfg['filtergraph']['graphname'],'log'])])))
			if cfg['debug'] and not sum([1 for handler in logger.handlers if 'baseFilename' in dir(handler) and handler.baseFilename == self.logfile]):
				import codecs
				import cfgparser
				f = codecs.open(self.logfile,'w','utf8')
				inistring = cfgparser.readrawinistring(os.path.abspath(os.path.expanduser(os.sep.join([cfg['filtergraph']['graphs'],cfg['filtergraph']['graphname'],'config.ini']))))
				f.write('###### Raw config.ini file #######:\x0a%s\x0a##################################\x0a'%(inistring))
				useropts = cfgparser.getuseroptions(inistring)
				inistring = cfgparser.setuseroptions(inistring,useropts)
				f.write('##### Parsed config.ini file##### :\x0a%s\x0a##################################\x0a'%(inistring))
				del(codecs)
				del(cfgparser)
				formatter = logging.Formatter("%(asctime)s\t%(levelname)s\t%(name)s\t%(lineno)s\t%(funcName)s()\t%(message)s")
				fh = logging.FileHandler(self.logfile, mode='a')
				fh.setLevel(self.verbose)
				fh.setFormatter(formatter)
				logger.addHandler(fh)

			self.filtergraph = filtergraph()
			errors.extend(self.filtergraph.load(cfg['filtergraph']))

			self.isloaded = not errors
			if self.isloaded:
				logger.debug('%s\t%s',*['success','loaded'])
			else:
				logger.error('%s\t%s',*['failed','not loaded'])

		except KeyboardInterrupt:

			errors.append([__name__,'interrupt','keyboard'])

		return errors

	def unload(self):

		errors = []

		if not self.isloaded:
			logger.warn('%s\t%s',*['skip','not loaded'])
			return errors

		errors.extend(self.filtergraph.unload())
		del(self.filtergraph)

		self.isloaded = False
		logger.debug('%s\t%s',*['success','unloaded'])

		return errors

	def open(self,cfg,verbose=True):

		errors = []
		self.isopen = False

		try:

			self.verbose = logging.DEBUG if cfg['debug'] else cfg['verbose'] if cfg['verbose'] else logging.WARNING
			logger.setLevel(self.verbose)

			if not self.isloaded:
				logger.warn('%s\t%s',*['failed','not loaded'])
				return errors

			if self.isopen:
				logger.warn('%s\t%s',*['skip','already open'])
				return errors

			# create lock file name
			self.srcqueue = os.path.abspath(os.path.expanduser(os.sep.join([cfg['filtergraph']['graphs'],cfg['filtergraph']['graphname'],os.extsep.join([cfg['filtergraph']['graphname']])])))
			if os.name == 'nt':
				lockfile = 				os.sep.join([os.environ['ALLUSERSPROFILE'],'Application Data',realname,'%s.lock'%(cfg['filtergraph']['graphname'])])
				self.statusgraph = 		os.sep.join([os.environ['ALLUSERSPROFILE'],'Application Data',realname,'%s.%d'%(realname,os.getpid())])
				self.statusplugin = 	os.sep.join([os.environ['ALLUSERSPROFILE'],'Application Data',realname,'%s.plugin.%d'%(realname,os.getpid())])
			elif os.name == 'posix':
				lockfile = 				os.sep.join(['','var','tmp',realname,'%s.lock'%(cfg['filtergraph']['graphname'])])
				self.statusgraph = 		os.sep.join(['','var','tmp',realname,'%s.%d'%(realname,os.getpid())])
				self.statusplugin = 	os.sep.join(['','var','tmp',realname,'%s.plugin.%d'%(realname,os.getpid())])

			if cfg['cleanup']:
				if os.path.exists(lockfile): os.unlink(lockfile)
				if os.path.exists(self.srcqueue): os.unlink(self.srcqueue)
				if os.path.exists(self.srcqueue+os.extsep+'count'): os.unlink(self.srcqueue+os.extsep+'count')
				import tempfile
				from fnmatch import fnmatch
				for fname in[fname for fname in os.listdir(tempfile.gettempdir()) if fnmatch(fname,''.join(['~*','.tmp.lock']))]:
					os.unlink(os.sep.join([tempfile.gettempdir(),fname]))
					errors.append([__name__,'deleted',os.sep.join([tempfile.gettempdir(),fname])])
					logger.error('%s\t%s',*errors[-1][1:])
				self.isopen = False
				return errors

			try:
				os.makedirs(os.path.dirname(lockfile))
				# note 0o777 is octal notation
				os.chmod(os.path.dirname(lockfile),0o777)
			except OSError,e:
				if not e.errno == 17 and not e.errno == 1:
					errors.append([__name__,'access','can not write to \"%s\"'%(os.path.dirname(lockfile))])
					logger.error('%s\t%s',*errors[-1][1:])

			self.lock = flock(lockfile,False).acquire()
			if not self.lock:
				errors.append([__name__,'failed','session lock'])
				logger.error('%s\t%s',*errors[-1][1:])
				self.isopen = False
				return errors

			self.datetime = cf.timestamp()
			self.pid = os.getpid()
			if os.path.exists(self.srcqueue+os.extsep+'stop'): os.unlink(self.srcqueue+os.extsep+'stop')

			self.command = cfg['command']
			self.debug = cfg['debug']
			self.files = list(cfg['files'])
			self.hostname = cfg['hostname']
			self.markreadonly = cfg['markreadonly']
			self.resetreadonly = cfg['resetreadonly']
			self.restart = cfg['restart']
			self.runs = cfg['runs'] if cfg['runs'] and cfg['runs'] > 0 else 1
			self.umask = cfg['umask']
			self.usemanager = cfg['usemanager']
			self.progress = ProgressBar()

			self.filtergraph.srcqueue = self.srcqueue
			self.filtergraph.datetime = self.datetime
			self.filtergraph.debug = self.debug
			self.filtergraph.markreadonly = self.markreadonly
			self.filtergraph.resetreadonly = self.resetreadonly
			self.filtergraph.statusgraph = self.statusgraph
			self.filtergraph.statusplugin = self.statusplugin
			self.filtergraph.verbose = self.verbose


			cfg['filtergraph']['versions'] = cfg['versions']
			if cfg['versions']:
				sys.stderr.write('%s: %s\n'%(__name__,cfg['version']))

			errors.extend(self.filtergraph.open(cfg['filtergraph'],verbose))

			if cfg['versions']:

				self.isopen = False
				errors.append([__name__,'success','versions report'])
				logger.debug('%s\t%s',*errors[-1][1:])

			else:

				self.isopen = not errors
				if self.isopen:
					logger.debug('%s\t%s',*['success','opened'])
				else:
					logger.error('%s\t%s',*['failed','not opened'])

		except KeyboardInterrupt:

			errors.append([__name__,'interrupt','keyboard'])

		return errors

	def close(self,verbose=True):

		errors = []

		if not self.isopen:
			logger.warn('%s\t%s',*['skip','not open'])
			return errors

		if verbose:
			# update status file
			with open(self.statusgraph,'w') as f:
				f.write('%s\t%s\t%s\t%s\t%d\t%d\t%s\n'%(cf.timestamp(),self.filtergraph.graphname,'close()','start',self.progress.amount,self.progress.max,self.logfile))

		try:

			errors.extend(self.filtergraph.close())

		except KeyboardInterrupt:
			sys.stderr.write('\n')
			errors.append([__name__,'interrupt','keyboard'])

		if verbose:
			with open(self.statusgraph,'w') as f:
				f.write('%s\t%s\t%s\t%s\t%d\t%d\t%s\n'%(cf.timestamp(),self.filtergraph.graphname,'close()','%s'%('error' if errors else 'success'),self.progress.amount,self.progress.max,self.logfile))

		logger.debug('%s\t%s',*['error' if errors else 'success','closed'])

		if not self.command == 'stop' and not errors:
			if os.path.exists(self.srcqueue+os.extsep+'stop'): os.unlink(self.srcqueue+os.extsep+'stop')

		# don't delete self.statusgraph if it's a server version
		if os.path.splitext(os.path.basename(sys.argv[0]))[0].startswith('domy'):
			if os.path.exists(self.statusgraph): os.unlink(self.statusgraph)
			try:
				# fails with error if other processes made self.statusgraph
				os.rmdir(os.path.dirname(self.statusgraph))
			except:
				pass

		self.lock.release()

		self.isopen = False

		return errors

	def stop(self):
		open(self.srcqueue+os.extsep+'stop','w')
		logger.info('%s\t%s',*['stop','stop file created'])
		# update status file
		with open(self.statusgraph,'w') as f:
			f.write('%s\t%s\t%s\t%s\t%d\t%d\t%s\n'%(cf.timestamp(),self.filtergraph.graphname,'stop()','progress',self.progress.amount,self.progress.max,self.logfile))
		return 250

	def run(self):
		'''run graphs by queue'''

		if not self.isopen:
			logger.warn('%s\t%s',*['skip','not open'])
			return 1

		if self.filtergraph.isrunning:
			logger.warn('%s\t%s',*['failed','filtergraph running'])
			return 1
		self.filtergraph.isrunning = True

		run = 1
		runs = self.runs * (self.filtergraph.distinct_flush_runs + 1)
		while run < runs+1:

			with codecs.open(self.srcqueue+os.extsep+'count','r','utf8') as fq:
				self.filtergraph.queuecount = int(fq.readline().strip())

			self.progress.reset(0,self.filtergraph.queuecount*runs,72,mode='fixed',char='#')

			with open(self.statusgraph,'w') as f:
				f.write('%s\t%s\t%s\t%s\t%d\t%d\t%s\n'%(cf.timestamp(),self.filtergraph.graphname,'run(%s)'%(run),'start',self.progress.amount,self.progress.max,self.logfile))
			logger.info('%s\t%d run %s of %s runs',*['start',self.progress.max,run,runs])
			sys.stdout.write("Processing%s: %s file set(s)\n"%(' run %s of %s runs'%(run,runs) if self.filtergraph.distinct_flush_runs else '',self.progress.max))
			sys.stdout.write(self.progress.bar)
			sys.stdout.flush()

			fq = codecs.open(self.srcqueue,'r','utf8')
			for filelist in fq:

				filelist = filelist.rstrip().split('\t')
				result = 0

				try:
					result = self.filtergraph.run(filelist,run)
					if result > 0: raise RuntimeError('')

					run += 1
					self.progress.increment_amount(runs)
					sys.stdout.write(self.progress.bar)
					sys.stdout.flush()

					# for status info to disk every 10 files
					with open(self.statusgraph,'w') as f:
						f.write('%s\t%s\t%s\t%s\t%d\t%d\t%s\n'%(cf.timestamp(),self.filtergraph.graphname,'run(%s)'%(run),'progress',self.progress.amount,self.progress.max,self.logfile))
					if os.path.exists(self.srcqueue+os.extsep+'stop'):
						raise KeyboardInterrupt('API, user request')

				except KeyboardInterrupt,e:
					result = 250 if str(e) else 251
					stop = 'stop' if str(e) else 'break'
					sys.stdout.write('\n')
					logger.info('%s\t%s',*[stop,'%s'%(str(e)) if str(e) else 'Ctrl-C, user request'])
					with open(self.statusgraph,'w') as f:
						f.write('%s\t%s\t%s\t%s\t%d\t%d\t%s\n'%(cf.timestamp(),self.filtergraph.graphname,'run(%s)'%(run),stop,self.progress.amount,self.progress.max,self.logfile))
					self.stop()
					run = runs+1
					break

				except RuntimeError,e:
					result = 249
					sys.stdout.write('\n')
					logger.error('%s\t%s - %s',*['error','%d of %d, %s'%(self.progress.amount,self.progress.max,os.sep.join(self.filtergraph.filelist)),str(e)])
					with open(self.statusgraph,'w') as f:
						f.write('%s\t%s\t%s\t%s\t%d\t%d\t%s\n'%(cf.timestamp(),self.filtergraph.graphname,'run(%s)'%(run),'error',self.progress.amount,self.progress.max,self.logfile))
					self.stop()
					run = runs+1
					break

				except Exception,e:
					result = 252
					sys.stdout.write('\n')
					logger.exception('%s\t%s - %s',*['exception','%d of %d, %s'%(self.progress.amount,self.progress.max,os.sep.join(self.filtergraph.filelist)),str(e)])
					if os.path.exists(os.path.dirname(self.statusgraph)):
						with open(self.statusgraph,'w') as f:
							f.write('%s\t%s\t%s\t%s\t%d\t%d\t%s\n'%(cf.timestamp(),self.filtergraph.graphname,'run(%s)'%(run),'exception',self.progress.amount,self.progress.max,self.logfile))
					self.stop()
					run = runs+1
					break

			fq.close()
			if result: break
			sys.stdout.write('\n')
			sys.stdout.write('Completed: %d file set(s)\n'%(self.progress.amount/runs))
			logger.info('%s\t%s',*['complete','%d of %d'%(self.progress.amount,self.progress.max)])
			with open(self.statusgraph,'w') as f:
				f.write('%s\t%s\t%s\t%s\t%d\t%d\t%s\n'%(cf.timestamp(),self.filtergraph.graphname,'run(%s)'%(run),'complete',self.progress.amount,self.progress.max,self.logfile))
			sys.stdout.flush()

		if self.usemanager and not self.restart:
			if os.path.exists(self.srcqueue): os.unlink(self.srcqueue)
			if os.path.exists(self.srcqueue+os.extsep+'count'): os.unlink(self.srcqueue+os.extsep+'count')

		self.filtergraph.isrunning = False

		return result

	def getinstalledmodels():
		return cf.getinstalledmodels()

	def getcorpusattributes(self,rootfolder='',roottype='',stage=''):
		if self.isloaded:
			return self.filtergraph.getcorpusattributes(rootfolder,roottype,stage)
		else:
			return []

	def getallinputs(self,rootfolder='',roottype='',stage=''):
		if self.isloaded:
			return self.filtergraph.getallinputs(rootfolder,roottype,stage)
		else:
			return []

	def buildqueue(self):
		'''writes all pathspecs to queue file list.'''

		if not self.isopen: return 1

		# update status file
		with open(self.statusgraph,'w') as f:
			f.write('%s\t%s\t%s\t%s\t%d\t%d\t%s\n'%(cf.timestamp(),self.filtergraph.graphname,'buildqueue()','start',0,0,self.logfile))
#			os.fsync(f)
		try:
			result = self.filtergraph.buildqueue(self.srcqueue,list(self.files))

		except KeyboardInterrupt,e:
			logger.info('%s\t%s',*['stop','Ctrl-C user request'])
			return 251

		logger.info('%s\t%d',*['count',result])

		return 0 if result else 253

def _setDataTypes(src,dest):
	for k in dest.keys():
		if not k in src: continue
		if type(src[k]) is list:
			if not type(dest[k]) is list: dest[k] = dest[k].strip(', ').split(',')
		elif type(src[k]) is bool:
			if not type(dest[k]) is bool: dest[k] = cf.zbool(dest[k])
		elif type(src[k]) is int:
			if not type(dest[k]) is int: dest[k] = int(dest[k])
		elif type(src[k]) is float:
			if not type(dest[k]) is float: dest[k] = float(dest[k])
		elif type(src[k]) is dict:
			if not type(dest[k]) is dict:
				dest[k] = [pair.strip(' -').strip().split(':') for pair in dest[k].replace('=',':').strip(',').split(',')]
				dest[k] = dict([[key.lower(),value] for key,value in [pair if len(pair) == 2 else pair+[''] for pair in dest[k]]])

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import parser\n\tfrom %s import manager\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0],
	os.path.splitext(os.path.basename(sys.argv[0]))[0],
	)

licensetxt=u'''CorpusFiltergraph™ v4.0
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
