#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#4.0.264 - version update

import sys
import os
import stat
import common as cf
import logging
import shutil
import codecs
from fnmatch import fnmatch
import glob
from progress_bar import ProgressBar

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))

class reader(object):

	cfg = {
		'encoding': 'utf8',
		'maxinput': 0,
		'null': False,
		'roottype': '',
		'stage': None,
		'chunk-file': 0,
		'rename-original': '',
		'version': '4.0.264',
		}
	encoding = 'utf8'
	maxinput = 0
	null = False
	roottype = ''
	seen = {}
	stage = None
	chunk_file = 0
	rename_original = ''
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		if cfg['stage']:
			self.stage = cfg['stage']
			self.p.stage = cfg['stage']
		else:
			self.errors.append([__name__,'missing','[%s] \"stage=<missing>\"'%(__name__.split(',')[-1])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		if cfg['roottype']:
			self.roottype = cfg['roottype'].strip(',').split(',')[0]
			self.p.roottype = cfg['roottype'].strip(',').split(',')[0]
		else:
			self.errors.append([__name__,'missing','[%s] \"roottype=<missing>\"'%(__name__.split(',')[-1])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.encoding = 'utf-8-sig' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']
		self.maxinput = 0 if cfg['maxinput'] < 1 else cfg['maxinput']
		self.null = cfg['null']
		self.seen = {}
		self.chunk_file = cfg['chunk-file']
		self.rename_original = cfg['rename-original']

	def run(self):
		'''Open the file and read contents into buffer'''
		for k in self.p.cfinput.keys():
			# 1) test and lock output file(s)
			if not self.p.cfoutput[k]['lock']:
				del(self.p.cfoutput[k])
				continue
			if self.null: continue
			# 2) open input files, read to buffers
			try:
				self.p.cfoutput[k]['tempbuff'] = [line.strip('\r\n') for line in codecs.open(self.p.cfinput[k]['tempname'],'r',self.encoding).readlines()]
			except IOError,e:
				logger.exception('%s\t%s - %s',*['failed',self.p.cfinput[k]['tempname'],str(e)])
				if k in self.p.cfinput: del self.p.cfinput[k]
				continue
			logger.debug('%s\t%s',*['file',self.p.cfinput[k]['tempname']])
		# 3) test & cleanup if fail
		if not len(self.p.cfoutput) == len(self.p.cfinput):
			for k in self.p.cfoutput.keys():
				del(self.p.cfoutput[k])
			logger.warn('%s\tinput/output %s',*['mismatch',os.sep.join(self.p.filelist)])
			return -2

	def flush(self):
		'''clear input object'''
		for k in self.p.cfinput.keys():
			del(self.p.cfinput[k])

	def close(self):
		return

	def getrecord(self):
		'''returns True on error'''
		filelist = self.p.filelist
		if tuple(self.p.filelist) in self.seen.keys(): return -2
		# use wildcard patterns to update self.p.languages
		if filelist[cf.kind].lower().startswith('tm') or filelist[cf.kind].lower().startswith('lm'):
			# update self.p.languages and self.p.toolchain based on filelist
			if sum([1 for pattern in self.p.targetpatterns if '?' in pattern or '*' in pattern]):
				listdirs = os.listdir(os.sep.join(filelist[:cf.rdrlang]))
				for pattern in self.p.targetpatterns:
					for lang in [dir for dir in listdirs if not dir[0] == '.' and fnmatch(dir,pattern)]:
						if not lang in self.p.languages: self.p.languages.append(lang)
				for lang in self.p.languages:
					for plugintype in ['aligner','filter','postfilter']:
						if not lang in self.p.toolchain[plugintype]: self.p.toolchain[plugintype][lang] = {}
		srckey = tuple(filelist[cf.stage+1:])
		src = os.sep.join(self.p.cfinput[srckey]['filelist'])
		# test and lock input file (usually srclang)
		if not self.p.cfinput[srckey]['lock']:
			del(self.p.cfinput[srckey])
			return -2
		# save local temp file
		if _getfile(src,self.p.cfinput[srckey]['tempname']):
			del(self.p.cfinput[srckey])
			return -2
		# add primary file to "seen" list
		self.seen[tuple(self.p.cfinput[srckey]['filelist'])] = None
		logger.debug('%s\t%s',*['source',src])
		# get matched tgtlang pair(s)
		if filelist[cf.kind].lower().startswith('tm'):
			# add aligned targets with aligner graphs
			for lang in [lang for lang in self.p.languages if not lang == filelist[cf.srclang]]:
				tgt = filelist[:cf.rdrlang]+[lang]+filelist[cf.basename:]
				tgtkey = tuple(tgt[cf.stage+1:])
				# test and lock input file (usually tgtlang)
				if not self.p.cfinput[tgtkey]['lock']:
					del(self.p.cfinput[tgtkey])
					continue
				# copy to local temp file
				if _getfile(os.sep.join(tgt),self.p.cfinput[tgtkey]['tempname']):
					del(self.p.cfinput[tgtkey])
					continue
				self.seen[tuple(tgt)] = None
				logger.debug('%s\t%s',*['target',os.sep.join(tgt)])

	def flagrecord(self,filelist):
		if os.path.exists(os.sep.join(filelist)):
			os.chmod(os.sep.join(filelist),stat.S_IRUSR+stat.S_IRGRP+stat.S_IROTH)

	def unflagrecord(self,filelist):
		if os.path.exists(os.sep.join(filelist)):
			os.chmod(os.sep.join(filelist),stat.S_IRUSR+stat.S_IWUSR+stat.S_IRGRP+stat.S_IROTH)

	def getcorpusattributes(self,rootfolder,roottype,stage):
		fs,root = self.getroot(rootfolder)
		path = os.sep.join([fs,root,roottype,stage])
		result = []
		for (dirpath, dirnames, filenames) in os.walk(path):
			for i in [i for i in reversed(range(len(dirnames))) if dirnames[i].startswith('.') or dirnames[i].startswith('tm') or dirnames[i].startswith('lm')]:
				dirnames.remove(dirnames[i])
				result.append(dirpath.replace(path,'').strip(os.sep).split(os.sep) if dirpath.replace(path,'').strip(os.sep) else [])
			if filenames: result.append(dirpath.replace(path,'').strip(os.sep).split(os.sep))
		return result

	def getallinputs(self,rootfolder,roottype,stage):
		fs,root = self.getroot(rootfolder)
		result = {}
		for corpusattribute in self.getcorpusattributes(rootfolder,roottype,stage):
			path = os.sep.join([fs,root,roottype,stage]+corpusattribute).rstrip(os.sep)
			pathlist = [fs,root,roottype,stage]+corpusattribute
			result[tuple(pathlist)] = []
			for dir in [dir for dir in os.listdir(path) if not dir.startswith('.')]:
				if os.path.isfile(os.sep.join([path,dir])):
					result[tuple(pathlist)].append(dir)
				elif dir.startswith('tm') or dir.startswith('lm'):
					for (dirpath, dirnames, filenames) in os.walk(os.sep.join([path,dir])):
						if not filenames: continue
						result[tuple(pathlist+dirpath.replace(os.sep.join(pathlist),'').lstrip(os.sep).split(os.sep))] = filenames
			if not result[tuple(pathlist)]: del(result[tuple(pathlist)])
		return result

	def buildqueue(self,srcqueue,searchpaths):
		progress = ProgressBar()
		sys.stdout.write("Paths:\n")
		for path in searchpaths:
			sys.stdout.write("   %s\n"%(os.sep.join(path)))
		totalsearchpaths = 1 if len(searchpaths) == 0 else len(searchpaths)
		progress.reset(0,totalsearchpaths,72,mode='fixed',char='#')
		sys.stdout.write('Searching...\n')
		sys.stdout.write(progress.bar)
		sys.stdout.flush()
		# search file system
		srctmnames = []
		tgttmnames = []
		othernames = []
		for path in searchpaths:
			logger.debug('%s\t%s',*['search',os.sep.join(path)])
			x = glob.glob(os.sep.join(path))
			try:
				srctmnames.extend([self.path2list(filelist,path[cf.root]) for filelist in x if path[cf.kind].lower().startswith('tm') and path[cf.srclang].lower() == path[cf.rdrlang].lower()])
				tgttmnames.extend([self.path2list(filelist,path[cf.root]) for filelist in x if path[cf.kind].lower().startswith('tm') and not path[cf.srclang].lower() == path[cf.rdrlang].lower()])
				othernames.extend([self.path2list(filelist,path[cf.root]) for filelist in x if not path[cf.kind].lower().startswith('tm')])
			except:
				othernames.extend([self.path2list(filelist,path[cf.root]) for filelist in x])
			progress.increment_amount()
			sys.stdout.write(progress.bar)
		# dedupe and check read-only
		srctmnames.sort()
		tgttmnames.sort()
		othernames.sort()
		filenames = srctmnames+othernames+tgttmnames
		keep = {}
		with codecs.open(srcqueue,'w','utf8') as fq:
			for filelist in [filelist for filelist in filenames if filelist]:
				if tuple(filelist) in keep: continue
				if not os.access(os.sep.join(filelist),os.W_OK):
					if self.p.resetreadonly:
						self.unflagrecord(filelist)
					else:
						logger.debug('%s\t%s',*['read-only',os.sep.join(filelist)])
						continue
				for flist in self.__chunk_lines(filelist):
					keep[tuple(flist)] = None
					fq.write('%s\n'%('\t'.join(flist).encode('utf8')))
					if self.maxinput and len(keep) == self.maxinput: break
				if self.maxinput and len(keep) == self.maxinput: break
		with open(srcqueue+os.extsep+'count','w') as fq:
			fq.write('%s\n'%(len(keep)))
		sys.stdout.write('\n')
		sys.stdout.write("Found: %d file set(s)\n"%(len(keep)) if len(keep) else "Files not found\n")
		sys.stdout.flush()
		return len(keep)

	def path2list(self,path,root):
		'''Converts filename to list of levels.'''
		if fnmatch(os.sep.join(path.split(os.sep)[1:root.count(os.sep)+2]),root):
			fs,workingpath = path.rstrip('\\/').split(os.sep.join(path.split(os.sep)[1:root.count(os.sep)+2]))
			return [fs.rstrip(os.sep),]+[os.sep.join(path.split(os.sep)[1:root.count(os.sep)+2]).strip(os.sep),]+workingpath.lstrip(os.sep).split(os.sep)
		else:
			return []

	def getroot(self,rootfolder):
		'''Splits rootfolder to file system and rootfolder'''
		folder = os.path.abspath(os.path.expanduser(self.normalizeroot(rootfolder)))
		if fnmatch(folder,'?:*'):
			# DOS drive letter : folder
			fss, folder = folder.split(':')
			fss = fss + ':'
		elif fnmatch(folder,'*:*'):
			# ssh hostname : folder
			fss, folder = folder.split(':')
			fss = fss + ':'
		elif fnmatch(folder,'\\\\*\\*'):
			# MS Windows UNC \ folder
			fss = '\\\\'+folder.lstrip(os.sep).split(os.sep)[0]
			folder = folder.replace(fss,'')
		else:
			# NFS mount / folder
			fss = ''
			folder = os.path.normpath(folder)
			if folder == '.': folder = '*'
		folder = folder.lstrip('\\/')
		return [unicode(fss), unicode(folder)]

	def normalizeroot(self,rootfolder):
		return rootfolder.replace('\\',os.sep).replace('/',os.sep)

	def __chunk_lines(self,filelist):
		if not self.chunk_file: return [filelist]
		lcount = cf.file_length(os.sep.join(filelist))
		if lcount <= self.chunk_file: return [filelist]
		sys.stdout.flush()
		sys.stderr.write('\n')
		sys.stderr.flush()
		return self.__chunkrecord(filelist,lcount)

	def __chunkrecord(self,filelist,lcount):
		'''returns True on error'''
		targetpatterns = list(self.p.targetpatterns)
		languages = list(self.p.languages)
		# use wildcard patterns to update languages
		if filelist[cf.kind].lower().startswith('tm') or filelist[cf.kind].lower().startswith('lm'):
			# update languages based on filelist
			if sum([1 for pattern in targetpatterns if '?' in pattern or '*' in pattern]):
				listdirs = os.listdir(os.sep.join(filelist[:cf.rdrlang]))
				for pattern in targetpatterns:
					for lang in [dir for dir in listdirs if not dir[0] == '.' and fnmatch(dir,pattern)]:
						if not lang in languages: languages.append(lang)
		# chunk source file
		queuefiles = self.__chunkfile(filelist,lcount)
		logger.debug('%s\t%s',*['source',os.sep.join(filelist)])
		# get matched tgtlang pair(s)
		if filelist[cf.kind].lower().startswith('tm'):
			tgtfiles = []
			# add aligned targets with aligner graphs
			for lang in [lang for lang in languages if not lang == filelist[cf.srclang]]:
				# chunk target file
				tgt = filelist[:cf.rdrlang]+[lang]+filelist[cf.basename:]
				tgtfiles.extend(self.__chunkfile(tgt,lcount))
				logger.debug('%s\t%s',*['target',os.sep.join(tgt)])
				if not len(queuefiles) % len(tgtfiles) and self.rename_original:
					os.rename(os.sep.join(tgt),'%s.%s'%(os.sep.join(tgt),self.rename_original.strip(os.extsep)))
			if not len(queuefiles) % len(tgtfiles) and self.rename_original:
				os.rename(os.sep.join(filelist),'%s.%s'%(os.sep.join(filelist),self.rename_original.strip(os.extsep)))
		elif self.rename_original:
			os.rename(os.sep.join(filelist),'%s.%s'%(os.sep.join(filelist),self.rename_original.strip(os.extsep)))
		return queuefiles

	def __chunkfile(self,filelist,lcount):

		fname = os.sep.join(filelist)
		ttlfiles = lcount/self.chunk_file+(1 if (lcount/self.chunk_file) % self.chunk_file else 0)
		chunk_lines = (int(lcount)/int(ttlfiles))+1
		filenames = []
		progress = ProgressBar()
		sys.stderr.write(': \"%s\" chunking %s lines into %s files ...\n'%(os.path.basename(fname),lcount,ttlfiles))
		progress.reset(0,lcount/self.chunk_file+1,72,mode='fixed',char='#')
		sys.stderr.write(progress.bar)
		sys.stderr.flush()

		linecnt = 1
		# open orig and chunk files
		orig_file = codecs.open(fname,'r',self.encoding)
		chunk_file = codecs.open('%s-%0*d%s'%(os.path.splitext(fname)[0],len(str(ttlfiles)),progress.amount,os.path.splitext(fname)[1]),'w','utf8')
		for line in orig_file:
			chunk_file.write('%s\x0a'%(line.rstrip('\r\n')))
			if not linecnt % chunk_lines:
				chunk_file.close()
				filenames.append(self.path2list(chunk_file.name,filelist[cf.root]))
				progress.increment_amount()
				sys.stderr.write(progress.bar)
				sys.stderr.flush()
				chunk_file = codecs.open('%s-%0*d%s'%(os.path.splitext(fname)[0],len(str(ttlfiles)),progress.amount,os.path.splitext(fname)[1]),'w','utf8')
			linecnt += 1
		chunk_file.close()
		filenames.append(self.path2list(chunk_file.name,filelist[cf.root]))
		progress.increment_amount()
		sys.stderr.write(progress.bar)
		sys.stderr.write('\n')
		sys.stderr.write(': \"%s\" done\n'%(os.path.basename(fname)))
		sys.stderr.flush()
		orig_file.close()

		# delete any left-over files from previous chunking
		for f in glob.glob('%s-*%s'%(os.path.splitext(fname)[0],os.path.splitext(fname)[1])):
			if not self.path2list(f,filelist[cf.root]) in filenames: os.unlink(f)

		return filenames

def _getfile(orig,dest):
	try:
		os.makedirs(os.path.dirname(dest))
	except OSError,e:
		if not e.errno == 17: return True
	try:
		shutil.copyfile(orig,dest)
	except IOError,e:
		return True

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import reader\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''CorpusFiltergraph™ v4.0
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
