#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#4.0.264 - version update

import sys
import os
import stat
import common as cf
import logging
import shutil
import codecs
from fnmatch import fnmatch
import glob
from progress_bar import ProgressBar

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))

class reader(object):

	cfg = {
		'encoding': 'utf8',
		'roottype': '',
		'stage': None,
		'version': '4.0.264',
		}
	encoding = 'utf8'
	roottype = ''
	seen = {}
	stage = None
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		if cfg['stage']:
			self.stage = cfg['stage']
			self.p.stage = cfg['stage']
		else:
			self.errors.append([__name__,'missing','[%s] \"stage=<missing>\"'%(__name__.split(',')[-1])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		if cfg['roottype']:
			self.roottype = cfg['roottype'].strip(',').split(',')[0]
			self.p.roottype = cfg['roottype'].strip(',').split(',')[0]
		else:
			self.errors.append([__name__,'missing','[%s] \"roottype=<missing>\"'%(__name__.split(',')[-1])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.encoding = 'utf-8-sig' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']

	def run(self):
		'''Open the file and read contents into buffer'''
		for k in self.p.cfinput.keys():
			# 1) test and lock output file(s)
			if not self.p.cfoutput[k]['lock']:
				del(self.p.cfoutput[k])
				continue
			logger.debug('%s\t%s',*['file',self.p.cfinput[k]['tempname']])

	def flush(self):
		'''clear input object'''
		for k in self.p.cfinput.keys():
			del(self.p.cfinput[k])

	def close(self):
		return

	def getrecord(self):
		'''returns True on error'''
		# use wildcard patterns to update self.p.languages
		srckey = tuple(self.p.filelist[cf.stage+1:])
		src = os.sep.join(self.p.cfinput[srckey]['filelist'])
		# test and lock input file (usually srclang)
		if not self.p.cfinput[srckey]['lock']:
			del(self.p.cfinput[srckey])
			return -2

	def flagrecord(self,filelist):
		return

	def unflagrecord(self,filelist):
		return

	def getcorpusattributes(self,rootfolder,roottype,stage):
		fs,root = self.getroot(rootfolder)
		path = os.sep.join([fs,root,roottype,stage])
		result = []
		for (dirpath, dirnames, filenames) in os.walk(path):
			for i in [i for i in reversed(range(len(dirnames))) if dirnames[i].startswith('.') or dirnames[i].startswith('tm') or dirnames[i].startswith('lm')]:
				dirnames.remove(dirnames[i])
				result.append(dirpath.replace(path,'').strip(os.sep).split(os.sep) if dirpath.replace(path,'').strip(os.sep) else [])
			if filenames: result.append(dirpath.replace(path,'').strip(os.sep).split(os.sep))
		return result

	def getallinputs(self,rootfolder,roottype,stage):
		fs,root = self.getroot(rootfolder)
		result = {}
		for corpusattribute in self.getcorpusattributes(rootfolder,roottype,stage):
			path = os.sep.join([fs,root,roottype,stage]+corpusattribute).rstrip(os.sep)
			pathlist = [fs,root,roottype,stage]+corpusattribute
			result[tuple(pathlist)] = []
			for dir in [dir for dir in os.listdir(path) if not dir.startswith('.')]:
				if os.path.isfile(os.sep.join([path,dir])):
					result[tuple(pathlist)].append(dir)
				elif dir.startswith('tm') or dir.startswith('lm'):
					for (dirpath, dirnames, filenames) in os.walk(os.sep.join([path,dir])):
						if not filenames: continue
						result[tuple(pathlist+dirpath.replace(os.sep.join(pathlist),'').lstrip(os.sep).split(os.sep))] = filenames
			if not result[tuple(pathlist)]: del(result[tuple(pathlist)])
		return result

	def buildqueue(self,srcqueue,searchpaths):
		progress = ProgressBar()
		sys.stdout.write("Paths:\n")
		for path in [self.getroot(self.p.rootfolder)+[self.roottype,self.stage[0]]]:
			sys.stdout.write("   %s\n"%(os.sep.join(path)))

		progress.reset(0,1,72,mode='fixed',char='#')
		sys.stdout.write('Bypassing...\n')
		sys.stdout.write(progress.bar)
		sys.stdout.flush()
		progress.increment_amount()
		sys.stdout.write(progress.bar)
		try:
			fq = open(srcqueue,'wb')
			fc = open(srcqueue+os.extsep+'count','wb')
		except OSError,e:
			logger.exception('%s\t%s',*['failed',str(e)])
			return True
		with codecs.open(srcqueue,'w') as fq:
			fq.write('%s\n'%('\t'.join(self.getroot(self.p.rootfolder)+[self.roottype,self.stage[0]])))
		with open(srcqueue+os.extsep+'count','w') as fq:
			fq.write('1\n')
		# close files
		fq.close()
		fc.close()
		sys.stdout.write('\n')
		sys.stdout.write("Found: 1 file set\n")
		sys.stdout.flush()
		return 1

	def path2list(self,path,root):
		'''Converts filename to list of levels.'''
		if fnmatch(os.sep.join(path.split(os.sep)[1:root.count(os.sep)+2]),root):
			fs,workingpath = path.rstrip('\\/').split(os.sep.join(path.split(os.sep)[1:root.count(os.sep)+2]))
			return [fs.rstrip(os.sep),]+[os.sep.join(path.split(os.sep)[1:root.count(os.sep)+2]).strip(os.sep),]+workingpath.lstrip(os.sep).split(os.sep)
		else:
			return []

	def getroot(self,rootfolder):
		'''Splits rootfolder to file system and rootfolder'''
		folder = os.path.abspath(os.path.expanduser(self.normalizeroot(rootfolder)))
		if fnmatch(folder,'?:*'):
			# DOS drive letter : folder
			fss, folder = folder.split(':')
			fss = fss + ':'
		elif fnmatch(folder,'*:*'):
			# ssh hostname : folder
			fss, folder = folder.split(':')
			fss = fss + ':'
		elif fnmatch(folder,'\\\\*\\*'):
			# MS Windows UNC \ folder
			fss = '\\\\'+folder.lstrip(os.sep).split(os.sep)[0]
			folder = folder.replace(fss,'')
		else:
			# NFS mount / folder
			fss = ''
			folder = os.path.normpath(folder)
			if folder == '.': folder = '*'
		folder = folder.lstrip('\\/')
		return [unicode(fss), unicode(folder)]

	def normalizeroot(self,rootfolder):
		return rootfolder.replace('\\',os.sep).replace('/',os.sep)

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import reader\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''CorpusFiltergraph™ v4.0
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
