#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#4.0.264 - version update

import sys
import os
import stat
import common as cf
import logging
import shutil
import codecs
from xml.etree.cElementTree import ElementTree
from fnmatch import fnmatch
import glob
from progress_bar import ProgressBar

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))

class reader(object):

	cfg = { 
		'adminlang': 'EN-US',
		'languagepairs': [['*','*']],
		'maxinput': 0,
		'roottype': '',
		'stage': [],
		'version': '4.0.264',
		'xmlns': '{http://www.w3.org/XML/1998/namespace}',
		}
	adminlang = 'EN-US'
	maxinput = 0
	seen = {}
	roottype = ''
	stage = []
	xmlns = '{http://www.w3.org/XML/1998/namespace}'
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		if cfg['stage'] == ['None']:
			self.stage = []
		else:
			self.stage = cfg['stage']
		if cfg['roottype']:
			self.roottype = cfg['roottype'].strip(',').split(',')[0]
			self.p.roottype = cfg['roottype'].strip(',').split(',')[0]
		else:
			self.errors.append([__name__,'missing','[%s] \"roottype=<missing>\"'%(__name__.split(',')[-1])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.adminlang = cfg['adminlang']
		self.maxinput = 0 if int(cfg['maxinput']) < 1 else int(cfg['maxinput'])
		self.seen = {}

		if not cfg['languagepairs']:
			self.errors.append([__name__,'missing','[%s] \"languagepairs= value required\"'%(__name__.split(',')[-1])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		if not self.p.languagepairs:
			# update language information
			self.p.cfg['languagepairs'] = cfg['languagepairs']
			self.p.cfg['sourcelanguages'], self.p.cfg['languages'] = zip(*self.p.cfg['languagepairs'])
			self.p.cfg['sourcelanguages'] = cf.dedupelist([item.lower() for item in self.p.cfg['sourcelanguages']])
			self.p.cfg['languages'] = cf.dedupelist([item.lower() for item in self.p.cfg['languages']]+self.p.cfg['sourcelanguages'])
			self.p.sourcelanguages = cf.dedupelist([lang for lang in self.p.cfg['sourcelanguages'] if not '*' in lang and not '?' in lang])
			self.p.languages = cf.dedupelist([lang for lang in self.p.cfg['languages'] if not '*' in lang and not '?' in lang])
			self.p.languagepairs = self.p.cfg['languagepairs']

		self.xmlns = cfg['xmlns']

	def run(self):
		'''run filter'''
		for k in self.p.cfinput.keys():
			fileprefix = self.p.cfoutput[k]['filelist'][:cf.stage+1]+self.p.cfinput[k]['filelist'][cf.stage:cf.basename]+[u'tm']
			del(self.p.cfoutput[k])
			# 1) parse file with ElementTree
			self.p.cfinput[k]['tempobj'] = ElementTree()
			self.p.cfinput[k]['tempobj'].parse(self.p.cfinput[k]['tempname'])
			# 2) parse all tu's. each tu is a phrase aligned across several languages
			counter = 0
			for tu in self.p.cfinput[k]['tempobj'].findall(self.tmxns+'body/'+self.tmxns+'tu'):
				srclang = tu.attrib.get(self.xmlns+'srclang', '').lower().replace('-','_').replace('*','-')
				srclang = tu.attrib.get('srclang', self.srclang).lower().replace('-','_').replace('*','-') if not srclang else srclang
				skiplangs = {}
				# create tu blank line for languages in the tmx file
				for lang in self.p.languages:
					if self.p.cfoutput[tuple(list(fileprefix+[srclang]+[lang]+[k[cf.basename]])[cf.stage+1:])]['lock']:
						self.p.cfoutput[tuple(list(fileprefix+[srclang]+[lang]+[k[cf.basename]])[cf.stage+1:])]['tempbuff'].append('')
					else:
						skiplangs[lang] = None
				# skip if all languages faile to lock
				if sorted(skiplangs.keys()) == sorted(self.p.languages): continue
				# process all tuv's
				for tuv in tu.findall(self.tmxns+'tuv'):
					# find tuv's language
					lang = tuv.attrib.get(self.xmlns+'lang', '').lower().replace('-','_').replace('*','-')
					lang = tuv.attrib.get('lang', '').lower().replace('-','_').replace('*','-') if not lang else lang
					# skip unmatched, unlocked languages
					if not lang in self.p.languages or lang in skiplangs.keys(): continue
					try:
						# add text for lang
						self.p.cfoutput[tuple(list(fileprefix+[srclang]+[lang]+[k[cf.basename]])[cf.stage+1:])]['tempbuff'][-1] = ' '.join(tuv.findtext(self.tmxns+'seg','').splitlines())
					except:
						logger.error('%s\t%s',*['failed','<tu> %d, lang %s in file %s is corrupted'])
					else:
						counter += 1
			logger.debug('%s\t%s',*['file',self.p.cfinput[k]['tempname']])
			del(self.p.cfinput[k]['tempobj'])

	def flush(self):
		'''clear input object'''
		for k in self.p.cfinput.keys():
			del(self.p.cfinput[k])

	def close(self):
		return

	def getrecord(self):
		'''returns True on error'''
		if tuple(self.p.filelist) in self.seen.keys(): return -2
		srckey = tuple(self.p.filelist[cf.stage+1:])
		src = os.sep.join(self.p.cfinput[srckey]['filelist'])
		# test and lock input file (usually srclang)
		if not self.p.cfinput[srckey]['lock']:
			del(self.p.cfinput[srckey])
			return -2
		self.p.cfinput[srckey]['tempobj'] = ElementTree()
		tmxfile = self.p.cfinput[srckey]['tempobj']
		try:
			tmxfile.parse(src)
		except Exception,e:
			sys.stderr.write('%s - %s xml/tmx structure is corrupted. Parse failed with error: %s\n'%('error',src,str(e)))
			logger.exception('%s\t%s - %s',*['error',src,str(e)])
			return -2
		if not tmxfile.getroot().tag.endswith('tmx'):
			sys.stderr.write('%s\t%s is not a valid tmx file\n'%('format',src))
			logger.error('%s\t%s is not a tmx file',*['format',src])
			return -2
		self.tmxns = tmxfile.getroot().tag[:-3]
		# 3) parse header, find default 'srclang'
		self.srclang = '_all_'
		header = tmxfile.find(self.tmxns+'header')
		if not header is None:
			self.srclang = header.attrib.get(self.xmlns+'srclang','').lower().replace('-','_').replace('*','-')
			if self.srclang == '': self.srclang = header.attrib.get('srclang',self.srclang).lower().replace('-','_').replace('*','-')
		if self.srclang == '': self.srclang = '_all_'
		# 4) parse all tu's, find all 'srclang'
		srclangs = {}
		for tu in tmxfile.findall(self.tmxns+'body/'+self.tmxns+'tu'):
			srclang = tu.attrib.get(self.xmlns+'lang', '').lower().replace('-','_').replace('*','-')
			srclang = tu.attrib.get('srclang', self.srclang).lower().replace('-','_').replace('*','-') if not srclang else srclang
			for srcpattern in self.p.sourcepatterns:
				if fnmatch(srclang,srcpattern): srclangs[srclang] = None
		self.p.sourcelanguages = sorted(srclangs.keys())
		# 5) parse all tuv's, find all 'rdrlang'
		rdrlangs = {}
		for tuv in tmxfile.findall(self.tmxns+'body/'+self.tmxns+'tu/'+self.tmxns+'tuv'):
			rdrlang = tuv.attrib.get(self.xmlns+'lang', '').lower().replace('-','_')
			rdrlang = tuv.attrib.get('lang', '').lower().replace('-','_') if not rdrlang else rdrlang
			for tgtpattern in self.p.targetpatterns:
				if fnmatch(rdrlang,tgtpattern): rdrlangs[rdrlang] = None
		self.p.languages = sorted(cf.dedupelist(rdrlangs.keys()+srclangs.keys()))
		# 6) save copy of tree to temp file for extraction/filtering
		tmxfile.write(self.p.cfinput[srckey]['tempname'],'UTF-8')
		del(self.p.cfinput[srckey]['tempobj'])
		del(tmxfile)
		# add tmx file to "seen" list
		self.seen[tuple(self.p.cfinput[srckey]['filelist'])] = None
		logger.debug('%s\t%s',*['file',src])

	def flagrecord(self,filelist):
		if os.path.exists(os.sep.join(filelist)):
			os.chmod(os.sep.join(filelist),stat.S_IRUSR+stat.S_IRGRP+stat.S_IROTH)

	def unflagrecord(self,filelist):
		if os.path.exists(os.sep.join(filelist)):
			os.chmod(os.sep.join(filelist),stat.S_IRUSR+stat.S_IWUSR+stat.S_IRGRP+stat.S_IROTH)

	def getcorpusattributes(self,rootfolder,roottype,stage):
		fs,root = self.getroot(rootfolder)
		path = os.sep.join([fs,root,roottype,stage])
		result = []
		for (dirpath, dirnames, filenames) in os.walk(path):
			for i in [i for i in reversed(range(len(dirnames))) if dirnames[i].startswith('.') or dirnames[i].startswith('tm') or dirnames[i].startswith('lm')]:
				dirnames.remove(dirnames[i])
				result.append(dirpath.replace(path,'').strip(os.sep).split(os.sep) if dirpath.replace(path,'').strip(os.sep) else [])
			if filenames: result.append(dirpath.replace(path,'').strip(os.sep).split(os.sep))
		return result

	def getallinputs(self,rootfolder,roottype,stage):
		fs,root = self.getroot(rootfolder)
		result = {}
		for corpusattribute in self.getcorpusattributes(rootfolder,roottype,stage):
			path = os.sep.join([fs,root,roottype,stage]+corpusattribute).rstrip(os.sep)
			pathlist = [fs,root,roottype,stage]+corpusattribute
			result[tuple(pathlist)] = []
			for dir in [dir for dir in os.listdir(path) if not dir.startswith('.')]:
				if os.path.isfile(os.sep.join([path,dir])):
					result[tuple(pathlist)].append(dir)
				elif dir.startswith('tm') or dir.startswith('lm'):
					for (dirpath, dirnames, filenames) in os.walk(os.sep.join([path,dir])):
						if not filenames: continue
						result[tuple(pathlist+dirpath.replace(os.sep.join(pathlist),'').lstrip(os.sep).split(os.sep))] = filenames
			if not result[tuple(pathlist)]: del(result[tuple(pathlist)])
		return result

	def buildqueue(self,srcqueue,searchpaths):
		progress = ProgressBar()
		sys.stdout.write("Paths:\n")
		for path in searchpaths:
			sys.stdout.write("   %s\n"%(os.sep.join(path)))
		totalsearchpaths = 1 if len(searchpaths) == 0 else len(searchpaths)
		progress.reset(0,totalsearchpaths,72,mode='fixed',char='#')
		sys.stdout.write('Searching...\n')
		sys.stdout.write(str(progress)+'\r')
		sys.stdout.flush()
		# search file system
		filenames = []
		for path in searchpaths:
			logger.debug('%s\t%s',*['search',os.sep.join(path)])
			filenames.extend([self.path2list(filelist,path[cf.root]) for filelist in glob.glob(os.sep.join(path))])
			progress.increment_amount()
			sys.stdout.write(str(progress)+'\r')
		# dedupe and check read-only
		filenames.sort()
		keep = {}
		with codecs.open(srcqueue,'w','utf8') as fq:
			for filelist in [filelist for filelist in filenames if filelist]:
				if tuple(filelist) in keep: continue
				if not os.access(os.sep.join(filelist),os.W_OK):
					if self.p.resetreadonly:
						self.unflagrecord(filelist)
					else:
						logger.debug('%s\t%s',*['read-only',os.sep.join(filelist)])
						continue
				keep[tuple(filelist)] = None
				fq.write('%s\n'%('\t'.join(filelist).encode('utf8')))
				if self.maxinput and len(keep) == self.maxinput: break
		with open(srcqueue+os.extsep+'count','w') as fq:
			fq.write('%s\n'%(len(keep)))
		sys.stdout.write('\n')
		sys.stdout.write("Found: %d file set(s)\n"%(len(keep)) if len(keep) else "Files not found\n")
		sys.stdout.flush()
		return len(keep)

	def path2list(self,path,root):
		'''Converts filename to list of levels.'''
		if fnmatch(os.sep.join(path.split(os.sep)[1:root.count(os.sep)+2]),root):
			fs,workingpath = path.rstrip('\\/').split(os.sep.join(path.split(os.sep)[1:root.count(os.sep)+2]))
			return [fs.rstrip(os.sep),]+[os.sep.join(path.split(os.sep)[1:root.count(os.sep)+2]).strip(os.sep),]+workingpath.lstrip(os.sep).split(os.sep)
		else:
			return []

	def getroot(self,rootfolder):
		'''Splits rootfolder to file system and rootfolder'''
		folder = os.path.abspath(os.path.expanduser(self.normalizeroot(rootfolder)))
		if fnmatch(folder,'?:*'):
			# DOS drive letter : folder
			fss, folder = folder.split(':')
			fss = fss + ':'
		elif fnmatch(folder,'*:*'):
			# ssh hostname : folder
			fss, folder = folder.split(':')
			fss = fss + ':'
		elif fnmatch(folder,'\\\\*\\*'):
			# MS Windows UNC \ folder
			fss = '\\\\'+folder.lstrip(os.sep).split(os.sep)[0]
			folder = folder.replace(fss,'')
		else:
			# NFS mount / folder
			fss = ''
			folder = os.path.normpath(folder)
			if folder == '.': folder = '*'
		folder = folder.lstrip('\\/')
		return [unicode(fss), unicode(folder)]

	def normalizeroot(self,rootfolder):
		return rootfolder.replace('\\',os.sep).replace('/',os.sep)

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import reader\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''CorpusFiltergraph™ v4.0
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
