#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#4.0.264 - version update

import os
import sys
import subprocess
import common as cf
import logging

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True

class filter(object):

	cfg = {
		'args': {},
		'encoding': 'utf8',
		'exeoptions': '',
		'kw': {
			'bufsize': 0,
			'executable': None,
			'stdin': 'PIPE',
			'stdout': 'PIPE',
			'stderr': 'PIPE',
			'preexec_fn': None,
			'close_fds': False,
			'shell': False,
			'cwd': None,
			'env': None,
			'universal_newlines': False,
			'startupinfo': None,
			'creationflags': 0,
			},
		'script': '',
		'inputfile': '',
		'outputfile': '',
		'version': '4.0.264',
		}
	args = []
	encoding = 'utf8'
	kw = {}
	script = ''
	inputfile = ''
	outputfile = ''
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		self.args = dict(cfg['args'])
		self.kw = dict(cfg['kw'])
		if cfg['script']:
			self.script = self.p.decodepath('%s,%s'%(__name__.split(',')[1],cfg['script']),True)
			if not self.script:
				self.errors.append([__name__,'missing',cfg['script']])
				logger.error('%s\t%s',*self.errors[-1][1:])
			self.args[0] = self.script
			self.kw['cwd'] = os.path.dirname(self.script)
		else:
			self.script = None
		# convert args dict to args list
		self.args = [str(self.args[k]) for k in sorted(self.args.keys())]
		if 'executable' in cfg['kw'] and cfg['kw']['executable']:
			self.kw['executable'] = self.p.decodepath('%s,%s'%(__name__.split(',')[1],cfg['kw']['executable']),True)
			if not self.kw['executable']:
				self.errors.append([__name__,'missing',cfg['kw']['executable']])
				logger.error('%s\t%s',*self.errors[-1][1:])
			self.args.insert(0,cfg['exeoptions'] if cfg['exeoptions'] else '')
		else:
			self.kw['executable'] = None
		self.kw['stdin'] = subprocess.PIPE if cfg['kw']['stdin'] == 'PIPE' else cfg['kw']['stdin']
		self.kw['stdout'] = subprocess.PIPE if cfg['kw']['stdout'] == 'PIPE' else cfg['kw']['stdout']
		self.kw['stderr'] = subprocess.PIPE if cfg['kw']['stderr'] == 'PIPE' else cfg['kw']['stderr']

		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']
		self.inputfile = cfg['inputfile'].replace('%(rootfolder)s',self.p.rootfolder) if cfg['inputfile'] else self.inputfile
		self.outputfile = cfg['outputfile'].replace('%(rootfolder)s',self.p.rootfolder) if cfg['outputfile'] else self.outputfile
		if (self.inputfile and not self.outputfile) or (not self.inputfile and self.outputfile):
			self.errors.append([__name__,'invalid','[%s] inputfile=%s without outputfile= value'%(__name__,cfg['inputfile'])])
			logger.warn('%s\t%s',*self.errors[-1][1:])

	def run(self,k):
		global skipclose
		skipclose = not self.inputfile
		if self.inputfile: return

		try:
			subproc = subprocess.Popen(self.args,**self.kw)
		except Exception,e:
			raise RuntimeError('failed \"[%s]\" \"%s %s\" - %s'%(__name__,self.kw['executable'],' '.join(self.args),str(e)))
		txtout, txterr = subproc.communicate('\x0a'.join(self.p.cfoutput[k]['tempbuff']).encode(self.encoding) + '\n')
		self.p.cfoutput[k]['tempbuff'] = [line.decode(self.encoding) for line in txtout.splitlines()]

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		import codecs

		if not os.path.exists(self.inputfile):
			self.errors.append([__name__,'missing','[%s] %s'%(__name__,self.inputfile)])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return

		# make output folder
		try:
			os.makedirs(os.path.dirname(self.outputfile))
		except OSError,e:
			if not e.errno == 17:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		# open input and output files
		out = self.outputfile
		if out == self.inputfile:
			import tempfile
			fd,out = tempfile.mkstemp(suffix='.tmp', prefix='~', dir=self.p.tempdir)
			os.close(fd)
		try:
			o = codecs.open(out,'w',self.encoding)
			i = codecs.open(self.inputfile,'r',self.encoding)
		except:
			os.unlink(out)
			raise RuntimeError('Failed to open [%s] input/output files'%(__name__))

		sys.stderr.write('[%s] %s\n   Please wait'%(__name__,self.outputfile))
		cnt = 0

		try:
			subproc = subprocess.Popen(self.args,**self.kw)
		except Exception,e:
			raise RuntimeError('failed \"[%s]\" \"%s %s\" - %s'%(__name__,self.kw['executable'],' '.join(self.args),str(e)))

		try:
			# loop writes output line-by-line
			for line in i:
				subproc[tgtlang]['popen'].stdin.write('%s\n'%(line.encode(self.encoding).strip()))
				o.write('%s\n'%(subproc[tgtlang]['popen'].stdout.readline().rstrip('\r\n').decode(self.encoding)))
				cnt += 1
				if not cnt%5000: sys.stderr.write('.')
			sys.stderr.write('\n')
			# close input and output files
			i.close()
			o.close()

			if not out == self.outputfile:
				import shutil
				shutil.move(out,self.outputfile)

		except KeyboardInterrupt:
			os.unlink(out)
			raise KeyboardInterrupt()

		if not os.path.exists(self.outputfile):
			self.errors.append([__name__,'missing','[%s] %s'%(__name__,self.outputfile)])
			logger.error('%s\t%s',*self.errors[-1][1:])

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''CorpusFiltergraph™ v4.0
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
