#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#4.0.264 - version update

import sys
import os
import shutil
import re
from fnmatch import fnmatch
from flock import flock
import xml.etree.cElementTree as ElementTree
from StringIO import StringIO
import logging
import common as cf
import time
import codecs

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))

class writer(object):

	cfg = {
		'adminlang': 'EN-US',
		'deletezerolength': False,
		'encoding': 'UTF-8',
		'rendertypes': '.tmx',
		'rootfolder': '',
		'roottype': '',
		'safemode': False,
		'segtype': 'sentence',
		'stage' : '',
		'eol': 'linux',
		'version': '4.0.264',
		'versiondate': "20120116T12:00:00Z",
		}
	adminlang = 'EN-US'
	deletezerolength = False
	encoding = 'UTF-8'
	rendertypes = '.*'
	rootfolder = ''
	roottype = ''
	safemode = False
	segtype = 'sentence'
	stage = None
	eol = '\x0a'
	versiondate = ''
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		if cfg['stage'] == 'None':
			self.stage = ''
		else:
			self.stage = cfg['stage']
		if cfg['roottype']:
			self.roottype = cfg['roottype'].strip(',').split(',')[0]
			self.p.roottype = cfg['roottype'].strip(',').split(',')[0]
		else:
			self.errors.append([__name__,'missing','[%s] \"roottype=<missing>\"'%(__name__.split(',')[-1])])
			logger.error('%s\t%s',*self.errors[-1][1:])
		self.adminlang = cfg['adminlang']
		self.deletezerolength = cfg['deletezerolength']
		self.encoding = 'UTF-8' if 'utf8' in cfg['encoding'].lower().replace('-','') else 'UTF-16'
		self.rendertypes = os.extsep+cfg['rendertypes'].lstrip(os.extsep)
		self.rootfolder = self.normalizeroot(cfg['rootfolder'])
		self.safemode = cfg['safemode']
		if cfg['segtype'] in ['block', 'paragraph', 'sentence', 'phrase']:
			self.segtype = cfg['segtype']
		elif self.stage == 'pa':
			self.segtype = 'paragraph'
		elif self.stage == 'fa':
			self.segtype = 'block'
		else:
			self.segtype = 'sentence'
		self.eol = '\x0d\x0a' if 'win' in cfg['eol'].lower() else '\x0a' if 'linux' in cfg['eol'].lower() or 'unix' in cfg['eol'].lower() or 'posix' in cfg['eol'].lower() else cfg['eol']
		self.versiondate = cfg['versiondate']

	def run(self,fileobj,k=None,mode=None):
		'''saves buffer to TMX file'''
		# skip saving if buffer empty or already saved
		srckey = ()
		for kk in fileobj.keys():
			try:
				if not kk[cf.kind].lower().startswith('tm'):
					del(fileobj[kk])
					continue
			except:
				del(fileobj[kk])
				continue
			if kk[cf.srclang] == kk[cf.rdrlang]: srckey = kk

		# test all source/target buffers
		result = None
		if k:
			result = -2
		else:
			for k in fileobj.keys():
				if not len(fileobj[k]['tempbuff']) == len(fileobj[srckey]['tempbuff']):
					logger.warn('%s\t%s and %s sentence pairs not equal',*['skip',k[cf.srclang],k[cf.rdrlang]])
					result = -2
				else:
					r = self.__skiptest(fileobj,k)
					if r == -1:
						del(fileobj[k])
						r = None
					else:
						result = r or result
		if result: return result
		# exit if all buffers are emptied
		if len(fileobj) < 2 or not srckey:
			logger.warn('%s\t%s',*['skip','output buffers empty'])
			return -2

		# 1) get defaults
		tgtlangs = list(zip(*[item for item in fileobj.keys() if not item[cf.rdrlang] == item[cf.srclang]])[cf.rdrlang])
		null, patterns = cf.dedupelist(zip(*[item for item in self.p.languagepairs]))
		# 2) set location to save output, create key with new file extension
		filelist = fileobj[srckey]['filelist'][:cf.rdrlang]+[fileobj[srckey]['filelist'][cf.basename]]
		savekey = tuple(filelist[cf.stage+1:])
		if not fileobj[savekey]['lock']: return True
		fileobj[savekey]['filelist'] = filelist
		result = self.__skiptest(fileobj,savekey)
		if result == -2: return result

		# 3) set XML/TMX root
		s = '<?xml version=\"1.0\" ?><!DOCTYPE tmx SYSTEM \"tmx14.dtd\" ><tmx version=\"1.4\" />'

		fileobj[savekey]['tempobj'] = ElementTree.parse(StringIO(s)).getroot()

		# 4) Make XML/TMX Header
		fileobj[savekey]['header'] = ElementTree.SubElement(
			fileobj[savekey]['tempobj'], 'header',
				{
				'creationtool': ' '.join([licensetxt[0:18], __name__.split(',')[1],'(', self.versiondate,')']),
				'creationtoolversion': self.versiondate,
				'adminlang': self.adminlang.upper(),
				'segtype': self.segtype
				})
		# 5) Make XML/TMX body
		fileobj[savekey]['body'] = ElementTree.SubElement(fileobj[savekey]['tempobj'], 'body')
		gmt = time.gmtime()
		# 6) Make each XML/TMX tu (by line)
		for i in range(len(fileobj[srckey]['tempbuff'])):
			# Make source language XML/TMX tuv
			tu = ElementTree.SubElement(fileobj[savekey]['body'], 'tu', )
			tu.attrib['srclang'] = srckey[cf.srclang].upper().replace('_ALL_','*all*').replace("_","-")
			prop = ElementTree.SubElement(tu, 'prop', {'type':'Txt::Doc.'})
			prop.text = fileobj[srckey]['filelist'][cf.basename]
			# 7) Make target language XML/TMX tuv's
			count = 0
			for k in [k for k in fileobj.keys() if len(k) > 3 and not k[cf.rdrlang] == k[cf.srclang]]:
				engine = fileobj[k]['filelist'][cf.rdrlang].replace('%s_'%(k[cf.rdrlang]),'',1)
				# filter out extra languages that don't match the pattern
				for pattern in patterns:
					if not fnmatch(k[cf.rdrlang].lower(),pattern.lower()) or not len(fileobj[k]['tempbuff']) or not fileobj[k]['tempbuff'][i]:continue
					tuv = ElementTree.SubElement(tu, 'tuv', {'xml:lang':k[cf.rdrlang].upper().replace("_","-")})
					if not engine == fileobj[k]['filelist'][cf.rdrlang]: tuv.attrib['creationid'] = engine
					tuv.attrib['creationdate'] = '%4.4d%2.2d%2.2dT%2.2d:%2.2d:%2.2dZ'%( gmt[0],gmt[1],gmt[2],gmt[3],gmt[4],gmt[5] )	#YYYYMMDDThh:mm:ssZ
					seg = ElementTree.SubElement(tuv, 'seg')
					seg.text = fileobj[k]['tempbuff'][i]
					count += 1
			# 8) test for target languages
			if count:
				tuv = ElementTree.SubElement(tu, 'tuv', {'xml:lang':srckey[cf.rdrlang].upper().replace("_","-")})
				seg = ElementTree.SubElement(tuv, 'seg')
				seg.text = fileobj[srckey]['tempbuff'][i]
				count += 1
				prop.attrib['count'] = str(count)
			else:
				fileobj[savekey]['body'].remove(tu)
		# save XML/TMX to disk
		if len(fileobj[savekey]['body']):
			ElementTree.ElementTree(fileobj[savekey]['tempobj']).write(fileobj[savekey]['tempname'],encoding='UTF-8')
			contents = codecs.open(fileobj[savekey]['tempname'],'r','UTF-8').read().splitlines()
			contents[0] = contents[0].replace('\'','\"').replace('UTF-8',self.encoding)
			contents.insert(1,'<!DOCTYPE tmx SYSTEM \"tmx14.dtd\">')

			with open(fileobj[savekey]['tempname'],'wb') as f:
				if self.encoding == 'UTF-16': f.write('\xFF\xFE')
				f.write(self.eol.join(contents).encode(self.encoding))

		logger.debug('%s\t%s',*['file',fileobj[k]['tempname']])

	def flush(self,fileobj,k=None):
		'''move temp output files to server'''
		result = None
		if k:
			result = -2 if self.__mkfolder(fileobj,k) else self.__push(fileobj,k)
		else:
			# create destination folders (test write access)
			for k in fileobj.keys():
				result = self.__mkfolder(fileobj,k) or result
			# move tempfile to output path
			if not result:
				for k in fileobj.keys():
					result = self.__push(fileobj,k) or result
		for k in fileobj.keys():
			del(fileobj[k])
		return result

	def close(self):
		return

	def getroot(self,rootfolder):
		'''Splits rootfolder to file system and rootfolder'''
		folder = os.path.abspath(os.path.expanduser(rootfolder.rstrip().replace('/',os.sep).replace('\\',os.sep)))
		if fnmatch(folder,'?:*'):
			# DOS drive letter : folder
			fss, folder = folder.split(':')
			fss = fss + ':'
		elif fnmatch(folder,'*:*'):
			# ssh hostname : folder
			fss, folder = folder.split(':')
			fss = fss + ':'
		elif fnmatch(folder,'\\\\*\\*'):
			# MS Windows UNC \ folder
			fss = '\\\\'+folder.lstrip(os.sep).split(os.sep)[0]
			folder = folder.replace(fss,'')
		else:
			# NFS mount / folder
			fss = ''
			folder = os.path.normpath(folder)
			if folder == '.': folder = '*'
		folder = folder.lstrip('\\/')
		return [unicode(fss), unicode(folder)]

	def normalizeroot(self,rootfolder):
		return rootfolder.replace('\\',os.sep).replace('/',os.sep)

	def __skiptest(self,fileobj,k):
		filelist = list(fileobj[k]['filelist'])
		if not fnmatch(os.path.splitext(filelist[cf.basename])[1],self.rendertypes):
			filelist[cf.basename] = filelist[cf.basename]+self.rendertypes
		if os.path.exists(os.sep.join(filelist)) and self.safemode:
			logger.warn('%s\t%s - %s',*['skip','output already exists',os.sep.join(filelist),])
			return -2
		if not sum([len(line.strip()) for line in fileobj[k]['tempbuff']]):
			if not 'workbench' in filelist[cf.stage] and not 'extract' in filelist[cf.stage]:
				logger.warn('%s\t%s - %s',*['skip','empty data',fileobj[k]['tempname']])
			if os.path.exists(os.sep.join(filelist)) and self.deletezerolength:
				os.unlink(os.sep.join(filelist))
				try:
					os.removedirs(os.path.dirname(os.sep.join(filelist)))
				except OSError,e:
					if not e.errno == 39:
						logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,os.sep.join(filelist),])
			return -1

	def __mkfolder(self,fileobj,k):
		if os.path.exists(fileobj[k]['tempname']):
			try:
				os.makedirs(os.path.dirname(os.sep.join(fileobj[k]['filelist'])))
			except OSError,e:
				if not e.errno == 17:
					logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
					return -2

	def __push(self,fileobj,k):
		if not k in fileobj: return
		if not fnmatch(os.path.splitext(fileobj[k]['filelist'][cf.basename])[1],self.rendertypes):
			fileobj[k]['filelist'][cf.basename] = fileobj[k]['filelist'][cf.basename]+self.rendertypes
		if os.path.exists(fileobj[k]['tempname']):
			try:
				shutil.move(fileobj[k]['tempname'],os.sep.join(fileobj[k]['filelist']))
			except:
				return -2
			else:
				logger.debug('%s\t%s',*['file',os.sep.join(fileobj[k]['filelist'])])

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import reader\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''CorpusFiltergraph™ v4.0
Copyright © 2010-2012 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
